@@ -52,7 +52,7 @@
  *	dsk_ls_init
  *	main
  *
- * ========================================================================== */
+ * ========================================================================= */
 
 #include "libdesk.h"
 #include <stdlib.h>
@@ -70,7 +70,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-/* === Defines ============================================================== */
+/* === Defines ============================================================= */
 
 #if defined(S_IFLNK) && !defined(S_ISLNK)
 #define S_ISLNK(mode) (((mode) & S_IFMT) == S_IFLNK)
@@ -94,19 +94,26 @@
 #define TYPE_UNKN	5
 #define TYPE_LDIR       6
 
-#define TC_REG          "_"
-#define TC_EXEC	        "*"
-#define TC_DIR          "/"
-#define TC_LREG         "@"
-#define TC_LEXEC        "+"
-#define TC_LDIR         "-"
-#define TC_SOCK         "="
-#define TC_UNKN         "_"
+#define TC_REG          '_'
+#define TC_EXEC	        '*'
+#define TC_DIR          '/'
+#define TC_LREG         '@'
+#define TC_LEXEC        '+'
+#define TC_LDIR         '-'
+#define TC_SOCK         '='
+#define TC_UNKN         '_'
 
 #define USAGE		"Usage: dsk_ls [options] <path>"
 
+#ifdef MEM_DEBUG
+#define dsk_malloc(a, b) my_malloc((a), (b))
+#define dsk_free(a, b)   my_free((a), (b))
+#else
+#define dsk_malloc(a, b) malloc((a))
+#define dsk_free(a, b)   free((a))
+#endif
 
-/* === Typedefs ============================================================= */
+/* === Typedefs ============================================================ */
 
 typedef struct _file_entry {
     struct _file_entry *next;
@@ -132,7 +139,7 @@ int process_file_lists ();
 int free_file_lists ();
 void write_element _ANSI_ARGS_((FILE_ENTRY *fe, char *xbuf));
 
-/* === Global Variables ===================================================== */
+/* === Global Variables ==================================================== */
 
 /* these are set throught the cmd-line */
 /* They are reset on every call of dsk_ls by calling dsk_ls_init! */
@@ -161,8 +168,11 @@ FILE_LIST root_of_lists = {NULL, NULL, 0, NULL};
 
 int just_one_file = 0;
 
+int num_mallocs, num_frees;
+
 /* variables used when called from dsk_ls_and_tag_Cmd */
-int    o_iargc, o_start, o_end, o_current, o_maskc, o_maskinv;
+int    o_iargc, o_start, o_end, o_current, o_maskc;
+int    o_maskinv = 0, o_dotreg = 0;
 char **o_iargv, **o_maskv, *o_maskv_ptr;
 
 
@@ -170,6 +180,26 @@ char **o_iargv, **o_maskv, *o_maskv_ptr;
 /*                                 Functions			             */
 /* ========================================================================= */
 
+
+void *my_malloc (size_t size, char *caller)
+{
+    void *ptr;
+    
+    fprintf (stderr, "malloc %d bytes", size);
+    ptr = malloc (size);
+    fprintf (stderr, " --> %p (%s)\n", ptr, caller);
+    num_mallocs++;
+    return ptr;
+}
+
+void my_free (void *ptr, char *caller)
+{
+    fprintf (stderr, "free   %p (%s)\n", ptr, caller);
+    num_frees++;
+    free (ptr);
+}
+
+
 /* ============================================================================
  * Name: dsk_ls_init_and_parse_options
  * Desc: Called from dsk_ls_and_tag_Cmd, initializes internal variables,
@@ -184,6 +214,17 @@ char *path;
 {
     dsk_ls_init ();
 
+#ifdef MEM_DEBUG
+    {
+	int i = 0;
+
+	fprintf (stderr, "dsk_ls_init_... ");
+	while (i < argc)
+	    fprintf (stderr, "%s ", argv[i++]);
+	fprintf (stderr, "%s\n", path);
+    }
+#endif
+    
     strcpy (par_path, path);
     if (process_cmdline (argc, argv) != TCL_OK)
 	return (TCL_ERROR);
@@ -198,18 +239,20 @@ char *path;
  * In  : iargc, iargv (ignore masks), mask
  * Out : number of matching files read
  * ------------------------------------------------------------------------- */
-int dsk_ls_read_and_sort (iargc, iargv, maskc, maskv, mask_invert)
+int dsk_ls_read_and_sort (iargc, iargv, maskc, maskv, mask_invert, dot_reg)
 int iargc;
 char **iargv;
 int maskc;
 char **maskv;
 int mask_invert;
+int dot_reg;
 {
-    o_iargc = iargc;
-    o_iargv = iargv;
-    o_maskc = maskc;
-    o_maskv = maskv;
+    o_iargc   = iargc;
+    o_iargv   = iargv;
+    o_maskc   = maskc;
+    o_maskv   = maskv;
     o_maskinv = mask_invert;
+    o_dotreg  = dot_reg;
 
     read_file_lists();
     process_file_lists();
@@ -265,6 +308,11 @@ char *nametcbuf;
 void dsk_ls_cleanup ()
 {
     free_file_lists();
+#ifdef MEM_DEBUG
+    fprintf (stderr, "num_mallocs: %d, num_frees: %d %s\n",
+	     num_mallocs, num_frees,
+	     num_mallocs != num_frees ? "*unequal*" : "");
+#endif
 }
 
 
@@ -672,17 +720,17 @@ int free_file_lists ()
     while (fl)
     {
 	nfl = fl->next;
-	free (fl->fe_array);
+	dsk_free (fl->fe_array, "free_file_lists");
 	fe = fl->first;
 	while (fe)
 	{
 	    nfe = fe->next;
-	    free (fe->path);
-	    free (fe);
+	    dsk_free (fe->path, "free_file_lists 2");
+	    dsk_free (fe, "free_file_lists 3");
 	    fe = nfe;
 	}
 	if (fl != &root_of_lists)
-	    free (fl); 
+	    dsk_free (fl, "free_file_lists 4"); 
 
 	fl = nfl;
     }
@@ -850,16 +898,18 @@ FILE_LIST *fl;
     int i, num_dir = 0, num_oth = 0;
 
     /* fea1 will hold directories */
-    fea1 = (FILE_ENTRY **) malloc (
-				   fl->num_files * (sizeof (FILE_ENTRY *)));
+    fea1 = (FILE_ENTRY **) dsk_malloc (
+				   fl->num_files * (sizeof (FILE_ENTRY *)),
+				   "sort_by_type");
     if (!fea1) {
 	perror ("sort_by_type (fea1)");
 	exit (1);
     }
     
     /* fea2 will hold everything but directories */
-    fea2 = (FILE_ENTRY **) malloc (
-				   fl->num_files * (sizeof (FILE_ENTRY *)));
+    fea2 = (FILE_ENTRY **) dsk_malloc (
+				   fl->num_files * (sizeof (FILE_ENTRY *)),
+				   "sort_by_type 2");
     if (!fea2) {
 	perror ("sort_by_type (fea2)");
 	exit (1);
@@ -887,8 +937,8 @@ FILE_LIST *fl;
 		fea1, num_dir * sizeof (FILE_ENTRY *));
     }
 
-    free (fea1);
-    free (fea2);
+    dsk_free (fea1, "sort_by_type");
+    dsk_free (fea2, "sort_by_type 2");
 
 } /* sort_by_type */
 
@@ -910,8 +960,9 @@ FILE_LIST *fl;
     if (fl->num_files == 0)
         return;
     
-    fl->fe_array = (FILE_ENTRY **) malloc (fl->num_files
-					   * (sizeof (FILE_ENTRY *)));
+    fl->fe_array = (FILE_ENTRY **) dsk_malloc (fl->num_files
+					   * (sizeof (FILE_ENTRY *)),
+					       "process_file_list");
 
     if (!fl->fe_array) {
 	perror ("process_file_list");
@@ -995,15 +1046,16 @@ char *name;
     FILE_LIST *fl;
     char *np, *ls, buf[BUFSIZ];
     struct stat sb;
+    int ftype;
 
-    nfe = (FILE_ENTRY *) malloc (sizeof (FILE_ENTRY));
+    nfe = (FILE_ENTRY *) dsk_malloc (sizeof (FILE_ENTRY), "new_file_entry");
     if (!nfe) {
 	perror ("new_file_entry (nfe)");
 	exit (1);
     }
 
     /* alloc length * 2 to have space for escaping special characters */
-    np = (char *) malloc (strlen(name) * 2);
+    np = (char *) dsk_malloc (strlen(name) * 2, "new_file_entry 2");
     if (!np) {
 	perror ("new_file_entry (np)");
 	exit (1);
@@ -1019,49 +1071,75 @@ char *name;
     else
     	nfe->name = ls+1;
     
-    lstat (name, &(nfe->sb));
+    if (lstat (name, &(nfe->sb)) == 0)
+	ftype = (int) (nfe->sb.st_mode & S_IFMT);
+    else
+	ftype = -1;
 
-    switch (nfe->sb.st_mode & S_IFMT) {
+    nfe->tc[1] = '\0';
+    
+    switch (ftype) {
     case S_IFREG:    	
 	if (nfe->sb.st_mode & S_IEXEC) {
-	    nfe->type = TYPE_EXEC;
-	    strcpy (nfe->tc, TC_EXEC);
+	    char *c;
+	    int   have_dot = 0;
+
+	    if (o_dotreg)
+		for (c = nfe->name; *c && !have_dot; c++)
+		    if (*c == '.')
+			have_dot = 1;
+
+	    if (!have_dot)
+	    {
+		nfe->type = TYPE_EXEC;
+		nfe->tc[0] = TC_EXEC;
+	    }
+	    else
+	    {
+		nfe->type = TYPE_REG;
+		nfe->tc[0] = TC_REG;
+	    }
 	} else {
 	    nfe->type = TYPE_REG;
-	    strcpy (nfe->tc, TC_REG);
+	    nfe->tc[0] = TC_REG;
 	}
 	break;
     case S_IFDIR:    	
 	nfe->type = TYPE_DIR;
-	strcpy (nfe->tc, TC_DIR);
+	nfe->tc[0] = TC_DIR;
 	break;
     case S_IFLNK:    	
-	stat (name, &sb);
-	if (S_ISDIR(sb.st_mode)) {
-	    nfe->type = TYPE_LDIR;
-	    strcpy (nfe->tc, TC_LDIR);
+	if (stat (name, &sb) == 0) {
+	    if (S_ISDIR(sb.st_mode)) {
+		nfe->type = TYPE_LDIR;
+		nfe->tc[0] = TC_LDIR;
+	    } else {
+		nfe->type = TYPE_LINK;
+		if (sb.st_mode & S_IEXEC)
+		    nfe->tc[0] = TC_LEXEC;
+		else
+		    nfe->tc[0] = TC_LREG;
+	    }
 	} else {
-	    nfe->type = TYPE_LINK;
-	    if (sb.st_mode & S_IEXEC)
-		strcpy (nfe->tc, TC_LEXEC);
-	    else
-		strcpy (nfe->tc, TC_LREG);
+	    nfe->type = TYPE_UNKN;
+	    nfe->tc[0] = TC_UNKN;
 	}
 	break;
     case S_IFSOCK:   	
 	nfe->type = TYPE_SOCK;
-	strcpy (nfe->tc, TC_SOCK);
+	nfe->tc[0] = TC_SOCK;
 	break;
     default:		
 	nfe->type = TYPE_UNKN;
-	strcpy (nfe->tc, TC_UNKN);
+	nfe->tc[0] = TC_UNKN;
 	break;
     }
 
     if (nfe->type != TYPE_DIR && nfe->type != TYPE_LDIR)
     {
 	if (ignore_file (nfe)) {
-	    free (nfe);
+	    dsk_free (nfe->path, "new_file_entry");
+	    dsk_free (nfe, "new_file_entry 2");
 	    return NULL;
 	}
     }
@@ -1101,7 +1179,7 @@ FILE_LIST *new_file_list ()
 {
     FILE_LIST *nfl, *fl;
 
-    nfl = (FILE_LIST *) malloc (sizeof (FILE_LIST));
+    nfl = (FILE_LIST *) dsk_malloc (sizeof (FILE_LIST), "new_file_list");
     if (!nfl) {
 	perror ("new_file_list");
 	exit (1);
@@ -1157,16 +1235,17 @@ char *path;
 #ifdef LSDEBUG
     	fprintf (stderr, "Found file: %s\n", dir->d_name);
 #endif
-        if (!strcmp (".", dir->d_name) || !strcmp ("..", dir->d_name)) {
-    	    if (!par_no_points) {
-       	        strcpy (buffer, path);
-       	    	strcat (buffer, dir->d_name);
-		new_file_entry (buffer);
-    	    }
-        } else if (dir->d_name[0] == '.') {
-    	    if (par_show_all) {
-       	        strcpy (buffer, path);
-       	    	strcat (buffer, dir->d_name);
+        if (dir->d_name[0] == '.') {
+	    if (   (dir->d_name[1] == '\0')
+		|| (dir->d_name[1] == '.' && dir->d_name[2] == '\0')) {
+		if (!par_no_points) {
+		    strcpy (buffer, path);
+		    strcat (buffer, dir->d_name);
+		    new_file_entry (buffer);
+		}
+	    } else if (par_show_all) {
+		strcpy (buffer, path);
+		strcat (buffer, dir->d_name);
 		new_file_entry (buffer);
 	    }
     	} else {
@@ -1213,10 +1292,19 @@ int read_file_lists ()
 	    return (TCL_ERROR);
 	}
 	if (!S_ISDIR(sb.st_mode) || par_one_file) {
-	    /* par_path is a file */
-	    write_element (new_file_entry (par_path), NULL);
-	    just_one_file = 1;
-	    return (TCL_OK);
+	    FILE_ENTRY *fe;
+
+	    fe = new_file_entry (par_path);
+	    if (fe != NULL)
+	    {
+		write_element (fe, NULL);
+		dsk_free (fe->path, "read_file_lists");
+		dsk_free (fe, "read_file_lists 2");
+		just_one_file = 1;
+		return (TCL_OK);
+	    }
+	    else
+		return (TCL_ERROR);
 	} else
 	    strcat (par_path, "/");
     }
@@ -1405,6 +1493,9 @@ void dsk_ls_init ()
     o_iargv = NULL;
     o_maskc = 0;
     o_maskv = NULL;
+
+    num_mallocs = 0;
+    num_frees   = 0;
     
 } /* dsk_ls_init */
 
@@ -1423,6 +1514,17 @@ int argc;
 char *argv[];
 {
 
+#ifdef MEM_DEBUG
+    {
+	int i = 0;
+
+	fprintf (stderr, "dsk_ls ");
+	while (i < argc)
+	    fprintf (stderr, "%s ", argv[i++]);
+	fprintf (stderr, "\n");
+    }
+#endif
+
     dsk_ls_init ();
 
     if (process_cmdline (argc, argv) != TCL_OK)
@@ -1435,7 +1537,14 @@ char *argv[];
 	return (TCL_ERROR);
 
     if (just_one_file)
+    {
+#ifdef MEM_DEBUG
+	fprintf (stderr, "num_mallocs: %d, num_frees: %d %s\n",
+		 num_mallocs, num_frees,
+		 num_mallocs != num_frees ? "*unequal*" : "");
+#endif
 	return (TCL_OK);
+    }
 
 #ifdef LSDEBUG
     fprintf( stderr, "\nEntering process_file_lists ...\n");
@@ -1451,6 +1560,12 @@ char *argv[];
 
     free_file_lists();
 
+#ifdef MEM_DEBUG
+    fprintf (stderr, "num_mallocs: %d, num_frees: %d %s\n",
+	     num_mallocs, num_frees,
+	     num_mallocs != num_frees ? "*unequal*" : "");
+#endif
+
     return (TCL_OK);
     
 } /* dsk_ls_Cmd */
