@@ -41,15 +41,15 @@
 #endif
 #include "libdesk.h"
 
-#if (TCL_MAJOR_VERSION != 7)
-#error "tcl.h is not from version 7.x"
+#if (TCL_MAJOR_VERSION != 7) && (TCL_MAJOR_VERSION != 8)
+#error "tcl.h is not from version 7.x or 8.x"
 #endif
 
 #ifdef CATCH_SIGNALS
 void signal_handler (int signum);
 #endif
 
-#define TKDESK_VERSION "1.1"
+#define TKDESK_VERSION "1.2"
 
 Tcl_Interp *dsk_interp;
 
@@ -75,7 +75,6 @@ Tcl_Interp *interp;
     Tcl_CreateCommand (interp, "dskC_statfs", dsk_statfs_Cmd, NULL, NULL);
 
     /* these commands are for tcl optimization: */
-    Tcl_CreateCommand (interp, "dskC_ppflist", dsk_ppflist_Cmd, NULL, NULL);
     Tcl_CreateCommand (interp, "dskC_ls_and_tag", dsk_ls_and_tag_Cmd, NULL, NULL);
     Tcl_CreateCommand (interp, "dskC_init_ftags", dsk_init_ftags_Cmd, NULL, NULL);
 
