@@ -30,8 +30,11 @@
 
 #include "config.h"
 #include "ot.h"
+#ifdef HAVE_TCL8_0_H
+#include <tcl8.0.h>
+#else
 #include <tcl.h>
-
+#endif
 
 extern Tcl_Interp *dsk_interp;
 
@@ -59,7 +62,8 @@ int   dsk_ls_init_and_parse_options _ANSI_ARGS_((int argc, char **argv,
 					      char *path));
 int   dsk_ls_read_and_sort          _ANSI_ARGS_((int iargc, char **iargv,
 						 int maskc, char **maskv,
-						 int mask_invert));
+						 int mask_invert,
+						 int dot_reg));
 int   dsk_ls_next_file              _ANSI_ARGS_((char *elembuf,
 						 char *nametcbuf));
 void  dsk_ls_cleanup                _ANSI_ARGS_(());
