@@ -375,3 +375,117 @@ int *val;
     return Tcl_GetBoolean (interp, v_value, val);
 } /* Ot_GetBoolean */
 
+
+/* ============================================================================
+ * Name   : Ot_GetVar
+ * In     : interp - tcl interpreter
+ *          var - name of tcl variable
+ *          flags - OR'ed Tcl flags
+ * Out    : Pointer to var's value or NULL
+ * ------------------------------------------------------------------------- */
+char *Ot_GetVar (interp, var, flags)
+     Tcl_Interp *interp;
+     char       *var;
+     int         flags;
+{
+#if TCL_MAJOR_VERSION < 8
+    return Tcl_GetVar (interp, var, flags);
+#else
+    Tcl_Obj *obj;
+
+    obj = Tcl_ObjGetVar2 (interp, Tcl_NewStringObj (var, -1), NULL, flags);
+    if (obj != NULL)
+	return Tcl_GetStringFromObj (obj, NULL);
+    else
+	return NULL;
+#endif
+}
+
+
+/* ============================================================================
+ * Name   : Ot_GetVar2
+ * In     : interp - tcl interpreter
+ *          var - name of tcl variable
+ *          ind - "index" of array var
+ *          flags - OR'ed Tcl flags
+ * Out    : Pointer to var's value or NULL
+ * ------------------------------------------------------------------------- */
+char *Ot_GetVar2 (interp, var, ind, flags)
+     Tcl_Interp *interp;
+     char       *var;
+     char       *ind;
+     int         flags;
+{
+#if TCL_MAJOR_VERSION < 8
+    return Tcl_GetVar2 (interp, var, ind, flags);
+#else
+    Tcl_Obj *obj;
+
+    obj = Tcl_ObjGetVar2 (interp, Tcl_NewStringObj (var, -1),
+			  Tcl_NewStringObj (ind, -1), flags);
+    if (obj != NULL)
+	return Tcl_GetStringFromObj (obj, NULL);
+    else
+	return NULL;
+#endif
+}
+
+
+/* ============================================================================
+ * Name   : Ot_SetVar
+ * In     : interp - tcl interpreter
+ *          var - name of tcl variable
+ *          newval - new value of var
+ *          flags - OR'ed Tcl flags
+ * Out    : Pointer to var's value or NULL
+ * ------------------------------------------------------------------------- */
+char *Ot_SetVar (interp, var, newval, flags)
+     Tcl_Interp *interp;
+     char       *var;
+     char       *newval;
+     int         flags;
+{
+#if TCL_MAJOR_VERSION < 8
+    return Tcl_SetVar (interp, var, newval, flags);
+#else
+    Tcl_Obj *obj;
+
+    obj = Tcl_ObjSetVar2 (interp, Tcl_NewStringObj (var, -1), NULL,
+			  Tcl_NewStringObj (newval, -1), flags);
+    if (obj != NULL)
+	return Tcl_GetStringFromObj (obj, NULL);
+    else
+	return NULL;
+#endif
+}
+
+
+/* ============================================================================
+ * Name   : Ot_SetVar2
+ * In     : interp - tcl interpreter
+ *          var - name of tcl variable
+ *          newval - new value of var
+ *          flags - OR'ed Tcl flags
+ * Out    : Pointer to var's value or NULL
+ * ------------------------------------------------------------------------- */
+char *Ot_SetVar2 (interp, var, ind, newval, flags)
+     Tcl_Interp *interp;
+     char       *var;
+     char       *ind;
+     char       *newval;
+     int         flags;
+{
+#if TCL_MAJOR_VERSION < 8
+    return Tcl_SetVar2 (interp, var, ind, newval, flags);
+#else
+    Tcl_Obj *obj;
+
+    obj = Tcl_ObjSetVar2 (interp, Tcl_NewStringObj (var, -1),
+			  Tcl_NewStringObj (ind, -1),
+			  Tcl_NewStringObj (newval, -1), flags);
+    if (obj != NULL)
+	return Tcl_GetStringFromObj (obj, NULL);
+    else
+	return NULL;
+#endif
+}
