@@ -52,7 +52,12 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#ifdef HAVE_TCL8_0_H
+#include <tcl8.0.h>
+#include <tk8.0.h>
+#else
 #include <tk.h>
+#endif
 #include <X11/Xutil.h>
 /*#include "tkInt.h"
 #include "tkPort.h"*/
@@ -433,6 +438,9 @@ ImgXpmGetData(interp, masterPtr)
     if (Tcl_GetInt(interp, listArgv[3], &cpp) != TCL_OK) {
 	goto error;
     }
+    if (listArgv) {
+	ckfree((char*)listArgv);
+    }
 
     if (isAllocated) {
 	if (numLines != size[1] + ncolors + 1) {
@@ -463,9 +471,6 @@ ImgXpmGetData(interp, masterPtr)
     if (isAllocated && data) {
 	ckfree((char*)data);
     }
-    if (listArgv) {
-	ckfree((char*)listArgv);
-    }
 		   
     return TCL_ERROR;
 }
