@@ -31,7 +31,12 @@ static char rcsid[]=
 #include "config.h"
 #include <sys/types.h>
 #include <unistd.h>
+#ifdef HAVE_TCL8_0_H
+#include <tcl8.0.h>
+#include <tk8.0.h>
+#else
 #include <tk.h>
+#endif
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
 
@@ -661,7 +666,7 @@ int timeout;
 	token = Tcl_CreateTimerHandler(timeout, LockTimeout, 
 					(ClientData) pending);
 	while (!pending->response) {
-		Tcl_DoOneEvent(TCL_WINDOW_EVENTS);
+	    Tcl_DoOneEvent(TCL_ALL_EVENTS);
 	}
 	Tcl_DeleteTimerHandler(token);
 #else
@@ -724,7 +729,7 @@ int timeout;
 	token = Tcl_CreateTimerHandler(timeout, LockTimeout, 
 					(ClientData) pending);
 	while (!pending->response) {
-		Tcl_DoOneEvent(TCL_WINDOW_EVENTS);
+	    Tcl_DoOneEvent(TCL_ALL_EVENTS);
 	}
 	Tcl_DeleteTimerHandler(token);
 #else
@@ -952,7 +957,7 @@ int timeout;
 	token = Tcl_CreateTimerHandler(timeout, LockTimeout, 
 					(ClientData) pending);
 	while (!pending->response) {
-		Tcl_DoOneEvent(TCL_WINDOW_EVENTS);
+	    Tcl_DoOneEvent(TCL_ALL_EVENTS);
 	}
 	Tcl_DeleteTimerHandler(token);
 #else
