@@ -59,171 +59,235 @@
 #
 
 itcl_class dsk_FileList {
-    inherit dsk_Common
+    inherit dsk_Common Toplevel
 
     constructor {args} {
 	global tkdesk env
-
-	dsk_busy
 	
-	#
-	# Create a toplevel with this object's name
-	# (later accessible as $this-top):
-	#
-        set class [$this info class]
-        ::rename $this $this-tmp-
-        ::toplevel $this -class $class
-	wm withdraw $this
-        ::rename $this $this-top
-        ::rename $this-tmp- $this
+	if {$tkdesk(tcl_version) < 8.0} {
+	    Toplevel::constructor
+	    dsk_Common::constructor
+	}
 
-	frame $this.fMenu -bd 2 -relief raised
-	pack $this.fMenu -fill x
-	_create_menubar $this.fMenu
+	wm withdraw $top
+	dsk_busy
+
+	frame $top.fMenu -bd 2 -relief raised
+	pack $top.fMenu -fill x
+	_create_menubar $top.fMenu
 
 	# ---- Button Bar
 	if {[llength $tkdesk(small_button_bar)] > 0} {
-	    frame $this.fButtonBar -bd 1 -relief raised
-	    pack $this.fButtonBar -after $this.fMenu -fill x
-	    _create_button_bar $this.fButtonBar
+	    frame $top.fButtonBar -bd 1 -relief raised
+	    pack $top.fButtonBar -after $top.fMenu -fill x
+	    _create_button_bar $top.fButtonBar
 	}
 
 	# ---- Path Entry
 
-	frame $this.fEntry -bd 1 -relief raised
-	pack $this.fEntry -fill x
+	frame $top.fEntry -bd 1 -relief raised
+	pack $top.fEntry -fill x
 
-	entry $this.entry -bd 2 -relief sunken -width 5
-	bindtags $this.entry "$this.entry Entry All"
-	bind $this.entry <Double-1> "[bind Entry <Triple-1>]; break"
-	bind $this.entry <Any-Enter> {
+	entry $top.entry -bd 2 -relief sunken -width 5
+	bindtags $top.entry "$top.entry Entry All"
+	bind $top.entry <Double-1> "[bind Entry <Triple-1>]; break"
+	bind $top.entry <Any-Enter> {
 	    if $tkdesk(focus_follows_mouse) {focus %W}
 	}
-	bind $this.entry <Return> "
+	bind $top.entry <Return> "
 	    $this config -dir \[%W get\]
 	"
-	bind $this.entry <Tab> "focus $this"
-	bind $this.entry <3> "update idletasks; $this _path_popup %X %Y"
-	cb_bindForCompletion $this.entry <Control-Tab>
-    	blt_drag&drop target $this.entry \
-			handler text "dd_handle_text $this.entry 1"
+	bind $top.entry <Tab> "focus $top"
+	bind $top.entry <3> "update idletasks; $this _path_popup %X %Y"
+	cb_bindForCompletion $top.entry <Control-Tab>
+	cb_balloonHelp $top.entry {The path displayed in the current window. Change to visit another directory.  The right mouse button gives a popup menu of parent directories.}
+    	blt_drag&drop target $top.entry \
+			handler file "dd_handle_text $top.entry %v 1"
 
-	pack $this.entry -in $this.fEntry -side left -fill x \
+	pack $top.entry -in $top.fEntry -side left -fill x \
 		-expand yes -padx $pad -pady $pad -ipady 2
 
-	frame $this.fEntry.dis -width $tkdesk(pad)
-	pack $this.fEntry.dis -side right
-	menubutton $this.mbHist -bd 2 -relief raised \
+	frame $top.fEntry.dis -width $tkdesk(pad)
+	pack $top.fEntry.dis -side right
+	menubutton $top.mbHist -bd 2 -relief raised \
 		-bitmap @$tkdesk(library)/cb_tools/bitmaps/combo.xbm \
-		-menu $this.mbHist.menu
-	pack $this.mbHist -in $this.fEntry -side right -ipadx 2 -ipady 2
+		-menu $top.mbHist.menu
+	pack $top.mbHist -in $top.fEntry -side right -ipadx 2 -ipady 2
 
-	menu $this.mbHist.menu \
-		-postcommand "dir_history buildmenu $this.mbHist.menu ; update"
+	menu $top.mbHist.menu \
+		-postcommand "dir_history buildmenu $top.mbHist.menu ; update"
 	# add dummy entry to work around bug in pre Tk 4.0p2:
-	$this.mbHist.menu add command -label "dummy"
+	$top.mbHist.menu add command -label "dummy"
 	dir_history changed
-	bind $this.mbHist.menu <ButtonRelease-1> "
+	bind $top.mbHist.menu <ButtonRelease-1> "
 		set tkdesk(menu,control) 0
 		[bind Menu <ButtonRelease-1>]"
-	bind $this.mbHist.menu <Control-ButtonRelease-1> "
+	bind $top.mbHist.menu <Control-ButtonRelease-1> "
 		set tkdesk(menu,control) 1
 		[bind Menu <ButtonRelease-1>]"
 
 	# --- File Listbox
 	
-	dsk_FileListbox $this.flb -width $tkdesk(file_lb,minwidth) \
+	if {$tkdesk(tcl_version) >= 8.0} {
+	    set lbname ::[namespace tail $top.flb]
+	} else {
+	    set lbname $top.flb
+	}
+	dsk_FileListbox $lbname -viewer $this  \
+		-width $tkdesk(file_lb,minwidth) \
 		-height $tkdesk(file_lb,minheight) \
-		-viewer $this \
-		-pad $tkdesk(pad) -toplevel $this -notrivialdirs 0
-	pack $this.flb -fill both -expand yes
+		-pad $tkdesk(pad) -toplevel $top -notrivialdirs 0
+	pack [$lbname getFrame] -fill both -expand yes
 
 	# ---- create status bar
 	if $statbar {
-	    frame $this.fStatus -bd [expr $tkdesk(pad) > 0] -relief raised
-	    pack [set f $this.fStatus] -fill x
+	    frame $top.fStatus -bd [expr $tkdesk(pad) > 0] -relief raised
+	    pack [set f $top.fStatus] -fill x
 	    label $f.l -anchor w -font [cb_font $tkdesk(font,status)]
 	    pack $f.l -fill x -padx [expr $tkdesk(pad) / 2] \
 		    -pady [expr $tkdesk(pad) / 2]
 	}
 
 	# ---- bindings
-	bind $this <Any-Enter> \
+	bind $top <Any-Enter> \
 		"set tkdesk(active_viewer) $this; break"
-	bind $this <Tab> "focus $this.entry; break"
+	bind $top <Tab> "focus $top.entry; break"
 
-	bind $this <Control-i> "dsk_fileinfo; break"
-	bind $this <Control-f> "dsk_find_files; break"
-	bind $this <Control-n> "dsk_create file; break"	
-	bind $this <Control-d> "dsk_create directory; break"	
-	bind $this <Control-c> "dsk_copy; break"
-	bind $this <Control-r> "dsk_rename; break"
-	bind $this <Delete> "dsk_delete; break"
-	bind $this <Control-x> "dsk_ask_exec; break"
-	bind $this <Control-e> "dsk_edit; break"
-	bind $this <Control-o> "dsk_ask_dir; break"
-	bind $this <Control-p> "dsk_print; break"
-	bind $this <Control-Home> {dsk_cd ~; break}
-	bind $this <Control-Left> {dsk_cd [list [dir_history back]]; break}
-	bind $this <Control-Right> {dsk_cd [list [dir_history forward]]; break}
-	bind $this <Return> "dsk_openall; break"
-	bind $this <F1> "dsk_cbhelp $tkdesk(library)/doc/Guide howto"
+	bind $top <Control-i> "dsk_fileinfo; break"
+	bind $top <Control-f> "dsk_find_files; break"
+	bind $top <Control-n> "dsk_create file; break"	
+	bind $top <Control-d> "dsk_create directory; break"	
+	bind $top <Control-c> "dsk_copy; break"
+	bind $top <Control-r> "dsk_rename; break"
+	bind $top <Delete> "dsk_delete; break"
+	bind $top <Control-x> "dsk_ask_exec; break"
+	bind $top <Control-e> "dsk_edit; break"
+	bind $top <Control-o> "dsk_ask_dir; break"
+	bind $top <Control-p> "dsk_print; break"
+	bind $top <Control-w> "dsk_openwith; break"
+	bind $top <Control-Home> {dsk_cd ~; break}
+	bind $top <Control-Left> {dsk_cd [list [dir_history back]]; break}
+	bind $top <Control-Right> {dsk_cd [list [dir_history forward]]; break}
+	bind $top <Return> "dsk_openall; break"
+	bind $top <F1> "dsk_cbhelp $tkdesk(library)/doc/Guide howto"
 
-	bind $this <BackSpace> {dsk_cd ..}
-	bind $this <Up> "$this keynav up"
-	bind $this <Down> "$this keynav down"
-	bind $this <Prior> "$this keynav pgup"
-	bind $this <Next> "$this keynav pgdown"
-	bind $this <Home> "$this keynav first"
-	bind $this <End> "$this keynav last"
+	bind $top <BackSpace> {dsk_cd ..}
+	bind $top <Up> "$this keynav up"
+	bind $top <Down> "$this keynav down"
+	bind $top <Prior> "$this keynav pgup"
+	bind $top <Next> "$this keynav pgdown"
+	bind $top <Home> "$this keynav first"
+	bind $top <End> "$this keynav last"
+	bind $top <Menu> "$this keynav menu"
+	bind $top <F11> "$this keynav lbmenu"
 	foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
-	    bind $this <$l> "$this keynav $l"
-	    bind $this <Shift-$l> "$this keynav $l reverse"
+	    bind $top <$l> "$this keynav $l"
+	    bind $top <Shift-$l> "$this keynav $l reverse"
 	}
-	
+	foreach l {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z} {
+	    bind $top <Key-$l> "$this keynav [string tolower $l] reverse"
+	    #bind $top <Shift-$l> "$this keynav $l"
+	}
+	    
 	update idletasks
-	wm title $this "File List"
-	wm minsize $this $tkdesk(file_lb,minwidth) $tkdesk(file_lb,minheight)
+	wm title $top "File List"
+	wm minsize $top $tkdesk(file_lb,minwidth) $tkdesk(file_lb,minheight)
 	set w [cb_max $tkdesk(file_lb,minwidth) \
 		[expr $tkdesk(file_lb,width) - 10]]
-	dsk_place_window $this flist ${w}x$tkdesk(file_lb,height) 1
-	wm protocol $this WM_DELETE_WINDOW "$this close"
+	dsk_place_window $top flist ${w}x$tkdesk(file_lb,height) 1
+	wm protocol $top WM_DELETE_WINDOW "$this close"
 
 	if $tkdesk(fvwm) {
 	    # create the icon window
 	    # (this code is based upon the code posted by:
 	    # kennykb@dssv01.crd.ge.com (Kevin B. Kenny))
-	    toplevel $this-icon -bg [cb_col $tkdesk(color,icon)] \
+	    toplevel $top-icon -bg [cb_col $tkdesk(color,icon)] \
 		    -class Icon
-	    wm withdraw $this-icon
-	    label $this-icon.label \
+	    wm withdraw $top-icon
+	    label $top-icon.label \
 		    -image [dsk_image $tkdesk(icon,filelist)] -bd 0 \
 		    -bg [cb_col $tkdesk(color,icon)]
-	    pack $this-icon.label -ipadx 2 -ipady 2
-	    blt_drag&drop target $this-icon.label handler \
-		    file "$this _dd_drophandler"
+	    pack $top-icon.label -ipadx 2 -ipady 2
+	    blt_drag&drop target $top-icon.label handler \
+		    file "$this _dd_drophandler %v"
 	    update idletasks
-	    wm geometry $this-icon \
-		    [winfo reqwidth $this-icon]x[winfo reqheight $this-icon]
-	    wm protocol $this-icon WM_DELETE_WINDOW "$this delete"
-	    wm iconwindow $this $this-icon
+	    wm geometry $top-icon \
+		    [winfo reqwidth $top-icon]x[winfo reqheight $top-icon]
+	    wm protocol $top-icon WM_DELETE_WINDOW "$this delete"
+	    wm iconwindow $top $top-icon
 	} else {
-	    wm iconbitmap $this @$tkdesk(library)/images/xbm/filing_open.xbm
+	    wm iconbitmap $top @$tkdesk(library)/images/xbm/filing_open.xbm
 	}
 
 	eval config $args
 	dsk_sound dsk_new_filelist
 
-	if !$dontmap {
-	    wm deiconify $this
-	    tkwait visibility $this
-	    catch "lower $this .dsk_welcome"
+	if {$isFileSel} {
+	    #
+	    # We're a file selector
+	    #
+	    if $statbar {
+		destroy $top.fStatus
+	    }
+	    frame $top.fFileSel -bd [expr $tkdesk(pad) > 0] -relief raised
+	    pack [set f $top.fFileSel] -fill x
+
+	    label $f.lSel -text $fileSelLabel -anchor w
+	    pack $f.lSel -fill x -padx $tkdesk(pad)
+	    
+	    entry $f.eSel -bd 2 -relief sunken
+	    pack $f.eSel -fill x -padx $tkdesk(pad) -pady $tkdesk(pad) -ipady 2
+
+	    bindtags $f.eSel "$f.eSel Entry all"
+	    bind $f.eSel <Any-Enter> {
+		if $tkdesk(focus_follows_mouse) {focus %W}
+	    }
+
+	    frame $f.fButs
+	    pack $f.fButs -fill x
+
+	    button $f.bOK \
+		    -command "$this _fs_ok; $this delete" \
+		    -text " OK " -width 7
+	    button $f.bCancel \
+		    -command "$this _fs_cancel; $this delete" \
+		    -text "Cancel" -width 7
+	    pack $f.bOK $f.bCancel -in $f.fButs -side left -expand y \
+		    -padx $tkdesk(pad) -pady $tkdesk(pad) -ipady 1
+
+	    $lbname _add_binding <ButtonRelease-1> "$this _fs_select"
+	    $lbname _set_binding <Double-1> "
+	       catch \{after cancel \$tkdesk(_show_tip_id)\}
+               set tkdesk(file_lb,control) 0
+	       set tmpsc \$tkdesk(single_click)
+	       set tkdesk(single_click) 0
+	       set tkdesk(config_nostat) 1
+	       set tkdesk(_doubleclick) 1
+	       $this _fs_ok; $this delete;
+	       set tkdesk(config_nostat) 0
+	       catch {set tkdesk(single_click) \$tmpsc}
+	       catch {unset tmpsc}
+	       break
+	    "
+
+	    wm protocol $top WM_DELETE_WINDOW \
+		    "$this _fs_cancel; $this delete"
+	    wm title $top $fileSelTitle
+	    catch {wm transient $top [[dsk_active win] getToplevel]}
+	    raise $top
+	}
+	
+	if {!$dontmap} {
+	    wm deiconify $top
+	    tkwait visibility $top
+	    catch "lower $top .dsk_welcome"
 	    update
 	}
-	$this.entry icursor end
-	$this.entry xview end
 	
-	pack propagate $this 0
+	$top.entry icursor end
+	$top.entry xview end
+	
+	pack propagate $top 0
 	update
 
 	dsk_lazy
@@ -241,10 +305,8 @@ itcl_class dsk_FileList {
 	    }
 	}
 	
-	$this.flb delete
-        #after 10 "rename $this-top {}"		;# delete this name
-        catch {destroy $this}		;# destroy associated windows
-	catch {destroy $this-icon}
+	$top.flb delete
+	catch {destroy $top-icon}
 	set tkdesk(active_viewer) ""
 	foreach fv "[itcl_info objects -class dsk_FileViewer] \
 		[itcl_info objects -class dsk_FileList]" {
@@ -286,43 +348,55 @@ itcl_class dsk_FileList {
     }
 
     method keynav {what {opt ""}} {
+	set lb ::[$top.flb getLbObj]
 	switch $what {
 	    up {
-		$this.flb.dlb select up
+		$lb select up
 	    }
 	    down {
-		$this.flb.dlb select down
+		$lb select down
 	    }
 	    pgup {
-		$this.flb.dlb select pgup
+		$lb select pgup
 	    }
 	    pgdown {
-		$this.flb.dlb select pgdown
+		$lb select pgdown
 	    }
 	    first {
-		$this.flb.dlb select first
+		$lb select first
 	    }
 	    last {
-		$this.flb.dlb select last
+		$lb select last
+	    }
+	    menu {
+		set file [dsk_active sel]
+		if {$file != ""} {
+		    dsk_popup "" $file [winfo pointerx .] [winfo pointery .]
+		}
+	    }
+	    lbmenu {
+		set mb [$lbname getFrame].mb
+		tkMbPost $mb
+		tkMenuFirstEntry $mb.menu
 	    }
 	    default {
 		if {$opt == "reverse"} {
-		    $this.flb.dlb select letter $what 1
+		    $lb select letter $what 1
 		} else {
-		    $this.flb.dlb select letter $what
+		    $lb select letter $what
 		}
 	    }
 	}
-	$this.flb _tag_icons
+	$top.flb _tag_icons
     }
 
     method display_statbar {display} {
 	global tkdesk
-	
-	catch {destroy $this.fStatus}
+
+	catch {destroy $top.fStatus}
 	if $display {
-	    frame $this.fStatus -bd [expr $tkdesk(pad) > 0] -relief raised
-	    pack [set f $this.fStatus] -fill x
+	    frame $top.fStatus -bd [expr $tkdesk(pad) > 0] -relief raised
+	    pack [set f $top.fStatus] -fill x
 	    label $f.l -anchor w -font [cb_font $tkdesk(font,status)]
 	    pack $f.l -fill x -padx [expr $tkdesk(pad) / 2] \
 		    -pady [expr $tkdesk(pad) / 2]
@@ -345,23 +419,23 @@ itcl_class dsk_FileList {
 	global tkdesk
 
 	if {$dir == $directory} {
-	    # $this.flb config -directory $directory
-	    $this.flb refresh
+	    # $top.flb config -directory $directory
+	    $top.flb refresh
 	    if {$directory == "$tkdesk(trashdir)/"} {
 		if $tkdesk(fvwm) {
-		    if {[llength [$this.flb.dlb get]] == 0} {
-			$this-icon.label config \
+		    if {[llength [[$top.flb getLbObj] get]] == 0} {
+			$top-icon.label config \
 				-image [dsk_image $tkdesk(icon,trash-empty)]
 		    } else {
-			$this-icon.label config \
+			$top-icon.label config \
 				-image [dsk_image $tkdesk(icon,trash-full)]
 		    }
 		} else {
-		    if {[llength [$this.flb.dlb get]] == 0} {
-			wm iconbitmap $this \
+		    if {[llength [[$top.flb getLbObj] get]] == 0} {
+			wm iconbitmap $top \
 				@$tkdesk(library)/images/xbm/trashcan.xbm
 		    } else {
-			wm iconbitmap $this \
+			wm iconbitmap $top \
 				@$tkdesk(library)/images/xbm/trashcan_full.xbm
 		    }
 		}
@@ -375,9 +449,9 @@ itcl_class dsk_FileList {
 	switch -glob -- $cmd {
 	    get		{# return a list of all selected files
 		    set sfl ""
-		    set sl [$this.flb.dlb select get]
+		    set sl [[$top.flb getLbObj] select get]
 		    if {$sl != ""} {
-		        set fl [$this.flb.dlb get]
+		        set fl [[$top.flb getLbObj] get]
 			foreach s $sl {
 			    set file [lindex [split [lindex $fl $s] \t] 0]
 			    set file [string trimright $file " "]
@@ -391,17 +465,17 @@ itcl_class dsk_FileList {
 		    return $sfl
 			}
 	    clear	{# clear selection in listbox
-		    $this.flb.dlb select clear
+		    [$top.flb getLbObj] select clear
 			}
 	    X		{# copy selected filenames to X selection
 		    set sfl [$this select get] 
 		    if {$sfl != ""} {
 			if {$args == "names"} {
-			    selection handle $this "$this _retrieve_X_names"
+			    selection handle $top "$this _retrieve_X_names"
 			} else {
-			    selection handle $this "$this _retrieve_X"
+			    selection handle $top "$this _retrieve_X"
 			}
-		        selection own $this
+		        selection own $top
 		    } else {
 		        cb_info "Please select one or more files first."
 		    }
@@ -424,7 +498,8 @@ itcl_class dsk_FileList {
 
 	set dir [string trimright [$lb info public directory -value] "/"]
 	#set file [lindex [lindex [$lb.dlb get] [lindex $sel 0]] 0]
-	set file [string trimright [lindex [split [lindex [$lb.dlb get] \
+	set file [string trimright [lindex [split \
+		[lindex [[$lb getLbObj] get] \
 		[lindex $sel 0]] \t] 0] " "]
 	if $tkdesk(append_type_char) {
 	    set file [dskC_striptc $file]
@@ -444,7 +519,8 @@ itcl_class dsk_FileList {
 	}
 	set dir [string trimright [$lb info public directory -value] "/"]
 	#set file [lindex [lindex [$lb.dlb get] [lindex $sel 0]] 0]
-	set file [string trimright [lindex [split [lindex [$lb.dlb get] \
+	set file [string trimright [lindex [split \
+		[lindex [[$lb getLbObj] get] \
 		[lindex $sel 0]] \t] 0] " "]
 	::dsk_popup $lb "$dir/$file" $mx $my
     }
@@ -453,12 +529,13 @@ itcl_class dsk_FileList {
 	# do nothing
     }
 
-    method _dd_drophandler {} {
-	global DragDrop tkdesk
+    method _dd_drophandler {args} {
+	global tkdesk
 
 	catch "wm withdraw $tkdesk(dd_token_window)"
 	update
 	set dest $directory
+	set flist $args
 
 	if ![file writable $dest] {
 	    dsk_errbell
@@ -469,22 +546,22 @@ itcl_class dsk_FileList {
 	    }
 	}
 
-	#dsk_debug "Rec.: $DragDrop(file)"
+	#dsk_debug "Rec.: $flist"
 	#dsk_debug "dest: $dest"
 	if {[string first "$tkdesk(trashdir)/" $dest] == -1} {
-	    dsk_ddcopy $DragDrop(file) $dest
+	    dsk_ddcopy $flist $dest
 	} else {
 	    if !$tkdesk(quick_dragndrop) {
-		dsk_delete $DragDrop(file)
+		dsk_delete $flist
 	    } else {
 		if {!$tkdesk(file_lb,control) && !$tkdesk(really_delete)} {
-		    dsk_ddcopy $DragDrop(file) $dest
+		    dsk_ddcopy $flist $dest
 		} else {
 		    if {[cb_yesno "Really deleting! Are you sure that this is what you want?"] == 0} {
 			dsk_sound dsk_really_deleting
-			dsk_bgexec "$tkdesk(cmd,rm) $DragDrop(file)" \
-				"Deleting [llength $DragDrop(file)] File(s)..."
-			dsk_refresh "$DragDrop(file) $dest"
+			dsk_bgexec "$tkdesk(cmd,rm) $flist" \
+				"Deleting [llength $flist] File(s)..."
+			dsk_refresh "$flist $dest"
 		    }
 		}		    
 	    }
@@ -492,41 +569,37 @@ itcl_class dsk_FileList {
     }
 
     method _button_bar {} {
-	_create_button_bar $this.fButtonBar
+	_create_button_bar $top.fButtonBar
     }
 
-    proc id {{cmd ""}} {
-	if {$cmd == ""} {
-	    set i $id
-	    incr id
-	    return $i
-	} elseif {$cmd == "reset"} {
-	    set id 0
+    method _fs_select {} {
+	set sel [file tail [lindex [dsk_active sel] 0]]
+	if {[string length $sel] > 0} {
+	    set e $top.fFileSel.eSel
+	    $e delete 0 end
+	    $e insert end $sel
+	    $e icursor end
+	    $e xview end
 	}
     }
 
-    proc status_bar {display} {
+    method _fs_ok {} {
+	set e $top.fFileSel.eSel
+	set f [$e get]
+	if {[string length $f] > 0} {
+	    set fsSelFile $directory[$e get]
+	} else {
+	    set fsSelFile ""
+	}
+    }
+
+    method _fs_cancel {} {
+	set fsSelFile ""
+    }
+
+    method _configDir {} {
+	# backend private proc of public var "directory"
 	global tkdesk
-	
-	foreach fl [itcl_info objects -class dsk_FileList] {
-	    $fl display_statbar $display
-	}
-	if $display {
-	    dsk_status_ready
-	}
-	set statbar $display
-    }
-
-    #
-    # ----- Variables ---------------------------------------------------------
-    #
-
-    public dontmap 0
-
-    public dir {} {
-	global tkdesk
-
-	if ![winfo exists $this] return
 
 	set err [catch {set isdir [file isdirectory $dir]}]
 	if !$err {
@@ -555,6 +628,53 @@ itcl_class dsk_FileList {
 	}
     }
 
+    proc id {{cmd ""}} {
+	if {$cmd == ""} {
+	    set i $id
+	    incr id
+	    return $i
+	} elseif {$cmd == "reset"} {
+	    set id 0
+	}
+    }
+
+    proc status_bar {display} {
+	global tkdesk
+
+	foreach fl [itcl_info objects -class dsk_FileList] {
+	    $fl display_statbar $display
+	}
+	if $display {
+	    dsk_status_ready
+	}
+	set statbar $display
+    }
+
+    proc fileSelector {args} {
+	
+	set fs ::.flFileSel
+	catch {$fs delete}
+	eval dsk_FileList $fs -isFileSel 1 $args
+	set w [$fs getToplevel]
+	grab $w
+	set tkdesk(active_viewer) $fs
+	tkwait window $w
+	return $fsSelFile
+    }
+
+    #
+    # ----- Variables ---------------------------------------------------------
+    #
+
+    public dontmap 0
+
+    public dir {} {
+
+	if [winfo exists $top] {
+	    _configDir
+	}
+    }
+
     public directory {} {
 	global env tkdesk
 
@@ -566,24 +686,35 @@ itcl_class dsk_FileList {
 	set directory [dsk_canon_path $directory]
 	dsk_debug "Directory $directory"
 
-	$this.entry delete 0 end
-	$this.entry insert end [cb_tilde $directory collapse]
+	$top.entry delete 0 end
+	$top.entry insert end [cb_tilde $directory collapse]
 
 	# right-justify the text in the path entry:
-	if {[wm state $this] != "withdrawn"} {
-	    $this.entry icursor end
-	    $this.entry xview end
+	if {[wm state $top] != "withdrawn"} {
+	    $top.entry icursor end
+	    $top.entry xview end
 	}
 
 	# save position of scrollbar:
 	if {$last_directory != ""} {
 	    set ypos($last_directory) \
-		    [lindex [cb_old_sb_get $this.flb.dlb.sb] 2]
+		    [lindex [cb_old_sb_get \
+		    [::[$top.flb getLbObj] getFrame].sb] 2]
 	}
 
-	$this.flb config -directory $directory
+	if {[string first "$tkdesk(trashdir)/" $directory] > -1} {
+	    $top.flb config -dont_refresh 1
+	    if {$directory == "$tkdesk(trashdir)/"} {
+		$top.flb config -notrivialdirs 1 -showall 1
+	    } else {
+		$top.flb config -notrivialdirs 0 -showall 1
+	    }
+	    $top.flb config -dont_refresh 0
+	}
+	
+	$top.flb config -directory $directory
 	if [info exists ypos($directory)] {
-	    $this.flb.dlb _yview $ypos($directory)
+	    [$top.flb getLbObj] _yview $ypos($directory)
 	}
 
 	# add this directory to the path history:
@@ -597,66 +728,73 @@ itcl_class dsk_FileList {
 	set last_directory $directory
 	
 	if {[string first "$tkdesk(trashdir)/" $directory] > -1} {
-	    if {$directory == "$tkdesk(trashdir)/"} {
-		$this.flb config -notrivialdirs 1 -showall 1
-	    } else {
-		$this.flb config -notrivialdirs 0 -showall 1
-	    }
-	    pack forget $this.fEntry
-	    wm title $this Trash
-	    wm iconname $this Trash
+	    pack forget $top.fEntry
+	    wm title $top Trash
+	    wm iconname $top Trash
 	    if $tkdesk(fvwm) {
-		if {[llength [$this.flb.dlb get]] == 0} {
-		    $this-icon.label config \
+		if {[llength [[$top.flb getLbObj] get]] == 0} {
+		    $top-icon.label config \
 			    -image [dsk_image $tkdesk(icon,trash-empty)]
 		} else {
-		    $this-icon.label config \
+		    $top-icon.label config \
 			    -image [dsk_image $tkdesk(icon,trash-full)]
 		}
 	    } else {
-		if {[llength [$this.flb.dlb get]] == 0} {
-		    wm iconbitmap $this @$tkdesk(library)/images/xbm/trashcan.xbm
+		if {[llength [[$top.flb getLbObj] get]] == 0} {
+		    wm iconbitmap $top @$tkdesk(library)/images/xbm/trashcan.xbm
 		} else {
-		    wm iconbitmap $this \
+		    wm iconbitmap $top \
 			    @$tkdesk(library)/images/xbm/trashcan_full.xbm
 		}
 	    }
 	} else {
-	    if {[wm title $this] == "Trash"} {
-		$this.flb config -notrivialdirs 0
-		pack $this.fEntry -fill x -after $this.fButtonBar
+	    if {[wm title $top] == "Trash"} {
+		$top.flb config -notrivialdirs 0
+		pack $top.fEntry -fill x -after $top.fButtonBar
 	    }
 	    #set wt [file tail [string trimright $directory "/"]]/
-	    #wm title $this $wt
-	    #wm iconname $this $wt
+	    #wm title $top $wt
+	    #wm iconname $top $wt
 	    set_title
 	    if $tkdesk(fvwm) {
-		$this-icon.label config \
+		$top-icon.label config \
 			-image [dsk_image $tkdesk(icon,filelist)]
 	    } else {
-		wm iconbitmap $this @$tkdesk(library)/images/xbm/filing_open.xbm
+		wm iconbitmap $top @$tkdesk(library)/images/xbm/filing_open.xbm
 	    }
 	}
 
 	if !$height_set {
 	    set height_set 1
-	    set h [cb_max [cb_min [expr [llength [$this.flb.dlb get]] + 1] \
-		    $tkdesk(file_lb,height)] $tkdesk(file_lb,minheight)]
+	    if {$isFileSel} {
+		set off 9
+	    } else {
+		set off 1
+	    }
+	    set h [cb_max \
+		    [cb_min [expr [llength [[$top.flb getLbObj] get]] + $off] \
+		            $tkdesk(file_lb,height)] \
+		    $tkdesk(file_lb,minheight)]
 	    set w [cb_max $tkdesk(file_lb,minwidth) \
 		    [expr $tkdesk(file_lb,width) - 10]]
-	    wm geometry $this ${w}x$h
+	    wm geometry $top ${w}x$h
 	}
     }
 
     public pad {4} {
     }
+    public isFileSel 0
+    public fileSelTitle "Select File"
+    public fileSelLabel "Selection:"
 
     protected height_set 0
     protected last_directory ""
     protected ypos
+    protected lbname
 
     common id 0
     common statbar 0
+    common fsSelFile ""
 }
 
 
