@@ -102,6 +102,8 @@ if ![info exists tkdesk(geometry,dsk_appbar)] {
 }
 
 set tkdesk(toggle,appbar) 0
+set dsk_appbar(is_visible) 0
+
 proc _appbar_create {} {
     global tkdesk dsk_appbar cb_tools
 
@@ -125,6 +127,7 @@ proc _appbar_create {} {
     toplevel $t
     wm withdraw $t
 
+    set ipad $tkdesk(appbar,ipad)
     set side top
     set fside left
     set dsk_appbar(layout) vertical
@@ -152,7 +155,7 @@ proc _appbar_create {} {
     $m add command -label {Horizontal } -command {dsk_appbar layout horizontal}
     $m add command -label {Raise} -command {dsk_appbar raise}
     $m add command -label {Lower} -command {dsk_appbar lower}
-    $m add command -label {Hide} -command {dsk_appbar close}
+    $m add command -label {Hide AppBar} -command {dsk_appbar close}
     $m add separator
     $m add command -label {Move Top Left} -command {dsk_appbar move topleft}
     $m add command -label {Move Bottom Left} \
@@ -173,24 +176,26 @@ proc _appbar_create {} {
     #
     # Create Drag/Toggle Area
     #
-    button $t.bDrag -bitmap gray12 \
-	    -height 8 -width 8 -cursor top_left_arrow \
-	    -padx 0 -pady 0 -highlightthickness 0 \
-	    -command _appbar_toggle_vis
-    if {$dsk_appbar(layout) == "vertical"} {
-	pack $t.bDrag -padx 0 -pady 0 -fill x -side $side
-    } else {
-	pack $t.bDrag -padx 0 -pady 0 -fill y -side $side
-    }
-    set cc white
-    if {[winfo depth .] != 1} {
-	set cc [cb_col $tkdesk(color,drag)]
-    }
-    bindtags $t.bDrag "appbar $t.bDrag Button all"
-    bind $t.bDrag <ButtonPress-1> {
-	set dsk_appbar(oldx) [winfo rootx .dsk_appbar]
-	_appbar_bind_global_b1 %W %X %Y
-	#break
+    if {$tkdesk(appbar,dragger)} {
+	button $t.bDrag -bitmap gray12 \
+		-height 8 -width 8 -cursor top_left_arrow \
+		-padx 0 -pady 0 -highlightthickness 0 \
+		-command _appbar_toggle_vis
+	if {$dsk_appbar(layout) == "vertical"} {
+	    pack $t.bDrag -padx 0 -pady 0 -fill x -side $side
+	} else {
+	    pack $t.bDrag -padx 0 -pady 0 -fill y -side $side
+	}
+	set cc white
+	if {[winfo depth .] != 1} {
+	    set cc [cb_col $tkdesk(color,drag)]
+	}
+	bindtags $t.bDrag "appbar $t.bDrag Button all"
+	bind $t.bDrag <ButtonPress-1> {
+	    set dsk_appbar(oldx) [winfo rootx .dsk_appbar]
+	    _appbar_bind_global_b1 %W %X %Y
+	    #break
+	}
     }
     
     set count 0
@@ -265,7 +270,7 @@ proc _appbar_create {} {
 		    "special:mail" {
 			_appbar_mail $t.b$count
 			pack $t.b$count -in $t.f$fcount -side $side \
-				-fill both -ipadx 2 -ipady 2
+				-fill both -ipadx $ipad -ipady $ipad
 			bind $t.b$count <3> \
 				"_appbar_show_menu $count %X %Y"
 			bind $t.m$count <B3-Motion> \
@@ -275,13 +280,13 @@ proc _appbar_create {} {
 
 			if {$dsk_appbar(ddaction) != ""} {
 			    blt_drag&drop target $t.b$count handler file \
-			    "_appbar_dd_action [list $dsk_appbar(ddaction)]"
+			    "_appbar_dd_action [list $dsk_appbar(ddaction)] %v"
 			}
 		    }
 		    "special:dialup" {
 			_appbar_dialup $t.b$count $count
 			pack $t.b$count -in $t.f$fcount -side $side \
-				-fill both -ipadx 2 -ipady 2
+				-fill both -ipadx $ipad -ipady $ipad
 			bind $t.b$count <3> \
 				"_appbar_show_menu $count %X %Y"
 			bind $t.m$count <B3-Motion> \
@@ -291,14 +296,14 @@ proc _appbar_create {} {
 
 			if {$dsk_appbar(ddaction) != ""} {
 			    blt_drag&drop target $t.b$count handler file \
-			    "_appbar_dd_action [list $dsk_appbar(ddaction)]"
+			    "_appbar_dd_action [list $dsk_appbar(ddaction)] %v"
 			}
 		    }
 		    "special:trash" {
 			catch {unset dsk_appbar(trash,label)}
 			_appbar_trash $t.b$count
 			pack $t.b$count -in $t.f$fcount -side $side \
-				-fill both -ipadx 2 -ipady 2
+				-fill both -ipadx $ipad -ipady $ipad
 			if [info exists dsk_appbar(trash,label)] {
 			    pack $dsk_appbar(trash,label) \
 				    -in $t.f$fcount -side $side -fill both
@@ -313,7 +318,7 @@ proc _appbar_create {} {
 
 			if {$dsk_appbar(ddaction) != ""} {
 			    blt_drag&drop target $t.b$count handler file \
-			    "_appbar_dd_action [list $dsk_appbar(ddaction)]"
+			    "_appbar_dd_action [list $dsk_appbar(ddaction)] %v"
 			}
 		    }
 		}
@@ -329,7 +334,7 @@ proc _appbar_create {} {
 			-command $dsk_appbar(defaction) \
 			-padx 0 -pady 0 -highlightthickness 0
 		pack $t.b$count -in $t.f$fcount -side $side -fill both \
-			 -ipadx 2 -ipady 2
+			 -ipadx $ipad -ipady $ipad
 		
 		bind $t.b$count <3> "_appbar_show_menu $count %X %Y"
 		bind $t.m$count <B3-Motion> "_appbar_motion $count %X %Y"
@@ -339,7 +344,7 @@ proc _appbar_create {} {
 
 		if {$dsk_appbar(ddaction) != ""} {
 		    blt_drag&drop target $t.b$count handler file \
-			    "_appbar_dd_action [list $dsk_appbar(ddaction)]"
+			    "_appbar_dd_action [list $dsk_appbar(ddaction)] %v"
 		}
 	    }
 
@@ -394,10 +399,12 @@ proc _appbar_toggle_vis {} {
 
     set t .dsk_appbar
     if $dsk_appbar(is_visible) {
-	if {$dsk_appbar(layout) == "vertical"} {
-	    $t.bDrag config -width [expr [winfo reqwidth $t] -4]
-	} else {
-	    $t.bDrag config -height [expr [winfo reqheight $t] -4]
+	if {[winfo exists $t.bDrag]} {
+	    if {$dsk_appbar(layout) == "vertical"} {
+		$t.bDrag config -width [expr [winfo reqwidth $t] -4]
+	    } else {
+		$t.bDrag config -height [expr [winfo reqheight $t] -4]
+	    }
 	}
 	for {set i 1} {$i <= $dsk_appbar(fcount)} {incr i} {
 	    pack forget $t.f$i
@@ -412,7 +419,9 @@ proc _appbar_toggle_vis {} {
 	for {set i 1} {$i <= $dsk_appbar(fcount)} {incr i} {
 	    pack $t.f$i -side $fside -fill both
 	}
-	$t.bDrag config -width 8 -height 8
+	if {[winfo exists $t.bDrag]} {
+	    $t.bDrag config -width 8 -height 8
+	}
 	set dsk_appbar(is_visible) 1
     }
 }
@@ -501,7 +510,7 @@ proc _appbar_handle_special_entry {m me} {
 		[expr [$m index last] + $inr] %W"
     } elseif {$me == "config"} {
 	$m add cascade -label "Configuration" -menu $m.cfg
-	menu [set tm $m.cfg]
+	menu [set tm $m.cfg] -tearoff 1
 	#menu $m.cfg
 	#$m.cfg add cascade -label "Edit Config Files" \
 	#	 -menu $m.cfg.edmenu
@@ -511,12 +520,23 @@ proc _appbar_handle_special_entry {m me} {
 	#menu [set tm $m.cfg.edmenu]
 	foreach cf $tkdesk(configfiles) {
 	    set cfs [format "%-12s" $cf]
-	    $tm add command -label "$cfs 	($tkdesk(configxpl,$cf))" \
+	    $tm add command -label "$cfs    ($tkdesk(configxpl,$cf))" \
 		    -command "dsk_edit_configs $cf"
 	}
+	$tm add separator
+	$tm add cascade -label "Reload" -menu [set rm $tm.remenu]
+	menu $rm -tearoff 1
+	foreach cf $tkdesk(configfiles) {
+	    $rm add command \
+		    -label "$cf" \
+		    -command "dsk_reread_config $cf"
+	}
+	$rm add separator
+	$rm add checkbutton -label " Reload Automatically" \
+		-variable tkdesk(auto_reload_conf)
+	
 	$tm add command -label "Find..." \
 		-command "dsk_find_files -path $tkdesk(configdir) -name [list $tkdesk(configfiles)] -type file"
-	$tm add separator
 	$tm add command -label "Colors..." \
 			-command "dsk_config_panel colors"
 	$tm add command -label "Fonts..." \
@@ -611,14 +631,14 @@ proc _appbar_unpost_parents {menu} {
     }
 }
 
-proc _appbar_dd_action {cmd} {
-    global DragDrop tkdesk
+proc _appbar_dd_action {cmd flist} {
+    global tkdesk
 
     catch "wm withdraw $tkdesk(dd_token_window)"
     update
     
     if {[string first %A $cmd] > -1} {
-	set cmd [string_replace $cmd %A $DragDrop(file)]
+	set cmd [string_replace $cmd %A $flist]
     } else {
 	set cmd [_expand_pc $cmd]
     }
@@ -796,12 +816,16 @@ proc _appbar_layout {orient} {
 	set side left
 	set fside top
 	set dsk_appbar(layout) horizontal
-	pack config .dsk_appbar.bDrag -padx 0 -pady 0 -fill y -side $side
+	if {[winfo exists .dsk_appbar.bDrag]} {
+	    pack config .dsk_appbar.bDrag -padx 0 -pady 0 -fill y -side $side
+	}
     } else {
 	set side top
 	set fside left
 	set dsk_appbar(layout) vertical
-	pack config .dsk_appbar.bDrag -padx 0 -pady 0 -fill x -side $side
+	if {[winfo exists .dsk_appbar.bDrag]} {
+	    pack config .dsk_appbar.bDrag -padx 0 -pady 0 -fill x -side $side
+	}
     }
 
     foreach obj [winfo children .dsk_appbar] {
@@ -922,7 +946,7 @@ proc _appbar_bind_special {widgets count mw} {
 	cb_balloonHelp $w $dsk_appbar(deflabel)
 	if {$dsk_appbar(ddaction) != ""} {
 	    blt_drag&drop target $w handler file \
-		    "_appbar_dd_action \"$dsk_appbar(ddaction)\""
+		    "_appbar_dd_action \"$dsk_appbar(ddaction)\" %v"
 	}
     }
 }
