@@ -146,6 +146,9 @@ proc dsk_read_config {} {
 	}
     }
 
+    # Save current mtime for automatic reloading, and desk update
+    set tkdesk(configdir_mtime) [file mtime $tkdesk(configdir)]
+    
     if ![file writable $tkdesk(configdir)] {
 	cb_alert "Will not be able to auto-save configuration (configuration directory is not writable)."
     }
@@ -163,6 +166,15 @@ proc dsk_read_config {} {
     set tkdesk(configxpl,Popups) {Popup Menus for Files etc.}
     set tkdesk(configxpl,Sounds) {Sounds to use for TkDesk events}
     set tkdesk(configxpl,System) {Fonts and Colors used by TkDesk, etc.}
+    #set tkdesk(configxpl,AppBar) {}
+    #set tkdesk(configxpl,ButtonBar) {}
+    #set tkdesk(configxpl,Commands) {}
+    #set tkdesk(configxpl,Directories) {}
+    #set tkdesk(configxpl,FileTags) {}
+    #set tkdesk(configxpl,Local) {}
+    #set tkdesk(configxpl,Popups) {}
+    #set tkdesk(configxpl,Sounds) {}
+    #set tkdesk(configxpl,System) {}
 
     read_System
     read_ButtonBar
@@ -212,8 +224,13 @@ proc dsk_edit_configs {args} {
 	}
     }
 
-    dsk_Editor .de[dsk_Editor :: id] -files $clist
-    #eval dsk_edit $clist
+    #dsk_Editor .de[dsk_Editor :: id] -files $clist
+    set ed $tkdesk(editor,cmd)
+    if [info exists tkdesk(conf_editor,cmd)] {
+	set tkdesk(editor,cmd) $tkdesk(conf_editor,cmd)
+    }
+    eval dsk_edit $clist
+    set tkdesk(editor,cmd) $ed
 }
 
 #
@@ -231,7 +248,8 @@ proc source_cfg {file} {
 
     set err [catch "source $file" errmsg]
     if $err {
-	dsk_lazy
+	# [todo] where does this dsk_lazy come from???
+	#dsk_lazy
 	dsk_errbell
 	cb_error "Error in file [file tail $file]: $errmsg"
     }
@@ -273,6 +291,7 @@ set tkdesk(font,status) -*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*
 set tkdesk(font,balloon)      -*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*
 set tkdesk(font,dialogs)      -*-times-medium-r-*-*-18-*-*-*-*-*-*-*
 set tkdesk(font,labels) -*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*
+set tkdesk(font,listboxes)    $tkdesk(font,labels)
 set tkdesk(font,buttons) -*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*
 set tkdesk(font,menubuttons)  $tkdesk(font,labels)
 set tkdesk(font,menus)        $tkdesk(font,menubuttons)
@@ -293,7 +312,9 @@ set tkdesk(editor,default_geometry) "80x25"
 set tkdesk(cmd,print) "lpr"
 set tkdesk(cmd,netscape) "netscape"
 set tkdesk(cmd,mail) "mail -s %s %a"
+set tkdesk(cmd,xterm) "xterm"
 set tkdesk(cmd,df) "df"
+set tkdesk(cmd,ps) "ps"
 set tkdesk(cmd,su,exec) {xterm -sb -T "%c" -n "%c" -e su root -c "%c"}
 set tkdesk(cmd,su,view) {xterm -sb -T "%c" -n "%c" -e pauseme su root -c "%c"}
 set tkdesk(focus_follows_mouse) 1
@@ -303,17 +324,27 @@ set tkdesk(editor,wrap) char
 set tkdesk(editor,real_tabs) 1
 set tkdesk(editor,tab_width) 8
 set tkdesk(editor,do_backups) 1
+set tkdesk(editor,load_once) 1
 set tkdesk(editor,cursor) black
+set tkdesk(editor,statbar) 1
+set tkdesk(editor,bindings) "classic"
+set tkdesk(editor,font) $tkdesk(font,entries)
 set tkdesk(desk_items,wm_managed) 0
 set tkdesk(desk_items,label_width) 12
 set tkdesk(dynamic_scrollbars) 1
 set tkdesk(use_old_modifiers) 0
 set tkdesk(color,annotation) gray55
+set tkdesk(side_of_scrollbars) right
+set tkdesk(focus_width) 1
+set tkdesk(scrollbar_width) 11
+set tkdesk(file_selector_type) "default"
+set tkdesk(deskdir) ~/desktop
 
 set tkdesk(title,browser) "%p"
 set tkdesk(title,list) "%d"
 set tkdesk(title,icon) "%d"
 
+
 proc read_System {} {
     global tkdesk tk_strictMotif
 
@@ -394,15 +425,17 @@ proc read_System {} {
 	}
     }
 
-    tk_setPalette \
-	    background [cb_col $tkdesk(color,background) white] \
-	    foreground [cb_col $tkdesk(color,foreground) black] \
-	    selectColor [cb_col $tkdesk(color,check_on)] \
-	    selectBackground [cb_col $tkdesk(color,select_bg)] \
-	    selectForeground [cb_col $tkdesk(color,select_fg)] \
-	    highlightBackground [cb_col $tkdesk(color,nofocus)] \
-	    highlightColor [cb_col $tkdesk(color,focus)] \
-	    insertBackground [cb_col $tkdesk(color,insert)]
+    if ![info exists tkdesk(config_done)] {
+	tk_setPalette \
+		background [cb_col $tkdesk(color,background) white] \
+		foreground [cb_col $tkdesk(color,foreground) black] \
+		selectColor [cb_col $tkdesk(color,check_on)] \
+		selectBackground [cb_col $tkdesk(color,select_bg)] \
+		selectForeground [cb_col $tkdesk(color,select_fg)] \
+		highlightBackground [cb_col $tkdesk(color,nofocus)] \
+		highlightColor [cb_col $tkdesk(color,focus)] \
+		insertBackground [cb_col $tkdesk(color,insert)]
+    }
 
     option add *background [cb_col $tkdesk(color,background) white] userDefault
     option add *foreground [cb_col $tkdesk(color,foreground) black] userDefault
@@ -437,12 +470,18 @@ proc read_System {} {
     option add *Button.font [cb_font $tkdesk(font,buttons)]
     option add *Menubutton.font [cb_font $tkdesk(font,menubuttons)]
     option add *Menu.font [cb_font $tkdesk(font,menus)]
+    option add *Listbox.font [cb_font $tkdesk(font,listboxes)]
+
+    _cb_help_proc:setfont \
+	    [cb_font $tkdesk(font,entries)] \
+	    [cb_font $tkdesk(font,listboxes)]
     
     option add *Menubutton.padX 3
     option add *Menubutton.padY 3
-    option add *Scrollbar.width 11
     option add *Scrollbar.borderWidth 2
-    option add *Menu.tearOff $tkdesk(tearoff-menus)
+    option add *Scrollbar.width $tkdesk(scrollbar_width)
+    option add *Entry.highlightThickness $tkdesk(focus_width) userDefault
+    option add *Button.highlightThickness $tkdesk(focus_width) userDefault
 
     if $tkdesk(focus_follows_mouse) {
 	tk_focusFollowsMouse
@@ -452,7 +491,7 @@ proc read_System {} {
 	set tkdesk(trashdir) $tkdesk(configdir)/.trash
     } else {
 	if ![file isdirectory $tkdesk(trashdir)] {
-	    set err [catch {mkdir $tkdesk(trashdir)} errmsg]
+	    set err [catch {exec mkdir $tkdesk(trashdir)} errmsg]
 	    if $err {
 		cb_error "Error: \"$errmsg\". Using default trash directory."
 		set tkdesk(trashdir) $tkdesk(configdir)/.trash
@@ -461,10 +500,29 @@ proc read_System {} {
     }
     set tkdesk(trashdir) [cb_tilde $tkdesk(trashdir) expand]
 
+    dsk_Listbox :: selcolor [cb_col $tkdesk(color,listsel)]
+    dsk_Listbox :: modifier $tkdesk(use_old_modifiers)
+    dsk_Listbox :: scrollbarside $tkdesk(side_of_scrollbars)
+    
     #set tkdesk(file_lb,minwidth) $tkdesk(file_lb,width)
     dsk_DeskItem :: set_width $tkdesk(desk_items,label_width)
 
+    # Backward compatibility:
+    set tkdesk(strict_motif) $tk_strictMotif
+
+    # Desktop dir handling:
+    if {[info exists tkdesk(deskdir)]} {
+	if {[string length $tkdesk(deskdir)] > 0} {
+	    set tkdesk(deskdir) [cb_tilde $tkdesk(deskdir) expand]
+	    if {![file isdirectory $tkdesk(deskdir)]} {
+		exec mkdir $tkdesk(deskdir)
+	    }
+	    set tkdesk(deskdir_mtime) [file mtime $tkdesk(deskdir)]
+	}
+    }
+    
     catch {destroy .dummy}
+    
     return
 }
 
@@ -548,21 +606,31 @@ set tkdesk(at_pointer) 1	;# Place dialogs at mouse pointer?
 set tkdesk(overwrite_always) 0	;# Overwrite existing files /wo asking?
 set tkdesk(really_delete) 0	;# Delete REALLY by default?
 set tkdesk(quick_dragndrop) 0   ;# don't ask when dropping files?
+set tkdesk(dot_regular) 0       ;# treat exec's containing dots as regulars?
 set tkdesk(sort_history) 1      ;# sort history menus?
 set tkdesk(use_sound) 0         ;# Use sound if available? (see file "Sounds")
 set tkdesk(in_browser) 0        ;# open new file windows always as browsers?
 set tkdesk(default_sort) name   ;# default sort style
 set tkdesk(ask_on_delete) 1     ;# ask even if not really deleting
+set tkdesk() 1	;# ask before exiting?
 set tkdesk(tkdesk_server) 0     ;# start TkDesk server
 set tkdesk(list_statbar) 0      ;# display status bar in file list windows?
+set tkdesk(tearoff-menus) 0     ;# 1 if menus should be tear-off-able
+set tkdesk(focus_follows_mouse) 1   ;# focus model to use
+set tkdesk(strict_motif) 1      ;# strict Motif look-and-feel?
+set tkdesk(auto_reload_conf) 1  ;# automatically reload modified conf files?
+set tkdesk(expand_dir_bookmarks) 1  ;# expand dirs in Bookmarks menu?
+set tkdesk(confirm_netscape) 1    ;# ask whether to start Netscape?
 
 set tkdesk(confvars) { \
 	long_listing show_all_files add_icons folders_on_top \
 	append_type_char single_click strip_home at_pointer overwrite_always \
 	really_delete quick_dragndrop sort_history use_sound default_sort \
-        ask_on_delete in_browser netscape_help tkdesk_server \
-	editor,auto_indent editor,wrap editor,do_backups \
-	editor,real_tabs editor,tab_width list_statbar appbar,autoraise
+        ask_on_delete ask_on_exit in_browser netscape_help tkdesk_server \
+	editor,auto_indent editor,wrap editor,do_backups editor,load_once \
+	editor,real_tabs editor,tab_width list_statbar appbar,autoraise \
+	tearoff-menus focus_follows_mouse strict_motif confirm_netscape \
+	auto_reload_conf editor,statbar expand_dir_bookmarks dot_regular
 }
 
 set tkdesk(autosave,history) 1
@@ -575,7 +643,7 @@ set cb_tools(balloon_help) 1	;# display neat/annoying help popups?
 set tkdesk(netscape_help) 0	;# display help thru netscape?
 
 proc read_Preferences {} {
-    global tkdesk
+    global tkdesk tk_strictMotif
 
     #
     # Read preferences
@@ -611,6 +679,10 @@ proc read_Preferences {} {
     }
     
     dsk_FileList :: status_bar $tkdesk(list_statbar)
+    dsk_FileListbox :: dotregular $tkdesk(dot_regular)
+
+    option add *Menu.tearOff $tkdesk(tearoff-menus)
+    set tk_strictMotif $tkdesk(strict_motif)
     
     return
 }
@@ -723,6 +795,7 @@ proc read_Popups {} {
 
 # some defaults:
 set tkdesk(appbar,wm_managed) 0
+set tkdesk(appbar,dragger) 0
 set tkdesk(appbar,max) 19
 set tkdesk(appbar,font,time) -*-courier-medium-r-*-*-10-*-*-*-*-*-*-*
 set tkdesk(appbar,font,weekday) -*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*
@@ -748,6 +821,7 @@ set tkdesk(appbar,dialup,down) next/PhoneTT.xpm
 set tkdesk(appbar,dialup,up)   next/PhoneTTOffhook.xpm
 set tkdesk(appbar,autoraise)   0
 set tkdesk(appbar,deffile) "AppBar"
+set tkdesk(appbar,ipad) 2
 
 proc read_AppBar {{fname "AppBar"}} {
     global tkdesk
@@ -1029,13 +1103,14 @@ proc save__layout {} {
 		    dsk_HistEntry} {
 		set obj_list [itcl_info objects -class $class]
 		foreach obj $obj_list {
-		    if ![winfo exists $obj] continue
+		    set top [$obj getToplevel]
+		    if ![winfo exists $top] continue
 		    switch $class {
 			dsk_FileViewer -
 			dsk_FileList {
-			    set state [wm state $obj]
+			    set state [wm state $top]
 			    if {$state == "normal" || $state == "iconic"} {
-				set geom [_shift_geom $obj]
+				set geom [_shift_geom $top]
 				puts -nonewline $fd "$class \{[$obj curdir]\} "
 				puts -nonewline $fd "$geom "
 				if {$class == "dsk_FileViewer"} {
@@ -1048,12 +1123,12 @@ proc save__layout {} {
 			}
 			dsk_DeskItem {
 			    puts -nonewline $fd "$class \{[$obj cget -file]\} "
-			    set geom [_shift_geom $obj]
+			    set geom [_shift_geom $top]
 			    puts $fd $geom
 			}
 			dsk_HistEntry {
 			    puts -nonewline $fd "$class "
-			    set geom [_shift_geom $obj]
+			    set geom [_shift_geom $top]
 			    puts $fd $geom
 			}
 		    }
@@ -1061,7 +1136,7 @@ proc save__layout {} {
 	    }
 	    foreach toplevel {dsk_jobs dsk_copy \
 		    dsk_delete dsk_appbar dsk_find_annotation dsk_bgexec \
-		    dsk_find_files} {
+		    dsk_find_files periodic} {
 		set geom ""
 		if [winfo exists .$toplevel] {
 		    set geom [_shift_geom .$toplevel]
@@ -1244,6 +1319,7 @@ proc dsk_restore_layout {} {
 		    }
 		    dsk_FileViewer $win -dir $dir \
 			    -num_lbs $nlb -dontmap 1
+		    set win [$win getToplevel]
 		    #wm withdraw $win
 		    wm geometry $win [lindex $l 2]
 		    if {[lindex $l 3] == "iconic" || $tkdesk(iconic)} {
@@ -1261,6 +1337,7 @@ proc dsk_restore_layout {} {
 		    dsk_progress "Creating a file list..."
 		    set win .dfl[dsk_FileList :: id]
 		    dsk_FileList $win -dir [lindex $l 1] -dontmap 1
+		    set win [$win getToplevel]
 		    #wm withdraw $win
 		    wm geometry $win [lindex $l 2]
 		    if {[lindex $l 3] == "iconic" || $tkdesk(iconic)} {
@@ -1274,13 +1351,17 @@ proc dsk_restore_layout {} {
 		    }
 		}
 	    	dsk_DeskItem {
-		    dsk_progress "Creating a desk item..."
-		    set win .di[dsk_DeskItem :: id]
 		    set itemname [lindex $l 1]
+		    if {![file exists $itemname]} {
+			continue
+		    }
 		    if [file isdirectory $itemname] {
 			set tkdesk(have_window) 1
 		    }
+		    dsk_progress "Creating a desk item..."
+		    set win .di[dsk_DeskItem :: id]
 		    dsk_DeskItem $win -file $itemname -dontmap 1
+		    set win [$win getToplevel]
 		    #wm withdraw $win
 		    set g [split [lindex $l 2] x+]
 		    wm geometry $win +[lindex $g 2]+[lindex $g 3]
@@ -1371,12 +1452,12 @@ proc dsk_reread_config {{cfgfile ""}} {
 		cb_image !reset
 		read_FileTags
 		foreach fl [itcl_info objects -class dsk_FileListbox] {
-		    if [winfo exists $fl] {
+		    if [winfo exists [$fl getFrame]] {
 			$fl refresh
 		    }
 		}
 		foreach di [itcl_info objects -class dsk_DeskItem] {
-		    if [winfo exists $di] {
+		    if [winfo exists [$di getToplevel]] {
 			$di refresh
 		    }
 		}
@@ -1539,11 +1620,11 @@ proc dsk_rebuild_cmdmenus {} {
 	foreach class "dsk_FileViewer dsk_FileList" {
 	    foreach t [itcl_info objects -class $class] {
 		if {$class == "dsk_FileViewer"} {
-		    set m $t.fMenu.mbCmds.menu
+		    set m [$t getToplevel].fMenu.mbCmds.menu
 		} else {
-		    set m $t.fMenu.mbOthers.menu.cmd
+		    set m [$t getToplevel].fMenu.mbOthers.menu.cmd
 		}
-		set fe [expr [$m cget -tearoff] ? 11 : 10]
+		set fe [expr [$m cget -tearoff] ? 12 : 11]
 		catch "$m delete $fe last"
 		foreach cmd $tkdesk(commands) {
 		    if {[llength $cmd] > 1} {
@@ -1643,7 +1724,7 @@ proc dsk_rebuild_dirmenus {} {
     if [info exists tkdesk(directories)] {
 	foreach class "dsk_FileViewer dsk_FileList" {
 	    foreach t [itcl_info objects -class $class] {
-		set m $t.fMenu.mbDirs.menu
+		set m [$t getToplevel].fMenu.mbDirs.menu
 		set fe [expr [$m cget -tearoff] ? 11 : 10]
 		catch "$m delete $fe last"
 		foreach mdir $tkdesk(directories) {
