@@ -106,7 +106,7 @@ proc dsk_delete {{files ""}} {
     bind $t.eFile <3> {dsk_popup %W [%W get] %X %Y}
     cb_bindForCompletion $t.eFile <Control-Tab>
 
-    blt_drag&drop target $t.eFile handler text "dd_handle_text $t.eFile 1"
+    blt_drag&drop target $t.eFile handler file "dd_handle_text $t.eFile %v 1"
 
     label $t.lFile
     if {$files == ""} {
@@ -381,12 +381,12 @@ proc dsk_empty_trash {} {
 	if {[cb_okcancel "Empty trash can?\nThis will delete ALL files in the trash can!"] == 0} {
 	    dsk_sound dsk_really_deleting
 	    if {$anum == 1} {set fll "File"} {set fll "Files"}
-	    set tlist ""
-	    foreach f $alist {
-		lappend tlist [string trimright \
-			$tkdesk(trashdir)/$f " "]
-	    }
-	    dsk_bgexec "$tkdesk(cmd,rm) $tlist" "Deleting $anum $fll..."
+	    #set tlist ""
+	    #foreach f $alist {
+	    #	 lappend tlist [string trimright \
+	    #		 $tkdesk(trashdir)/$f " "]
+	    #}
+	    dsk_bgexec "$tkdesk(cmd,rm) .* *" "Deleting $anum $fll..."
 	    dsk_refresh $tkdesk(trashdir)
 	}
     }
