@@ -63,7 +63,7 @@ if ![info exists tkdesk(geometry,dsk_bgexec)] {
     set tkdesk(geometry,dsk_bgexec) ""
 }
 
-proc dsk_bgexec {cmd label} {
+proc dsk_bgexec {cmd label args} {
     global dsk_bgexec tkdesk env
 
     set pad 8
@@ -88,8 +88,8 @@ proc dsk_bgexec {cmd label} {
     pack $t.fb -fill x
 
     button $t.bBreak -text "  Stop  " \
-	    -command "set dsk_bgexec(out$cnt) break ;\
-	    set dsk_bgexec(stat$cnt) {} ;\
+	    -command "set dsk_bgexec(stat$cnt) {} ;\
+	    set dsk_bgexec(out$cnt) break ;\
 	    destroy .dsk_bgexec$cnt"
     pack $t.bBreak -in $t.fb -side top -padx $pad -pady $pad -ipady 2
 
@@ -97,21 +97,22 @@ proc dsk_bgexec {cmd label} {
     wm title $t "Background #$cnt"
     wm protocol $t WM_DELETE_WINDOW {# ignore}
     if {$tkdesk(geometry,dsk_bgexec) == ""} {
-	cb_centerToplevel $t
+    	 cb_centerToplevel $t
+    	 wm deiconify $t
     } else {
-	set g [split $tkdesk(geometry,dsk_bgexec) +x]
-	set vw [winfo vrootwidth .]
-	if {$vw == ""} {set vw [winfo screenwidth .]}
-	set vh [winfo vrootheight .]
-	if {$vh == ""} {set vh [winfo screenheight .]}
-	set x [lindex $g 2]
-	while {$x < 0} {incr x $vw}
-	while {$x > $vw} {incr x -$vw}
-	set y [lindex $g 3]
-	while {$y < 0} {incr y $vh}
-	while {$y > $vh} {incr y -$vh}
-	wm geometry $t +$x+$y
-	wm deiconify $t
+    	 set g [split $tkdesk(geometry,dsk_bgexec) +x]
+    	 set vw [winfo vrootwidth .]
+    	 if {$vw == ""} {set vw [winfo screenwidth .]}
+    	 set vh [winfo vrootheight .]
+    	 if {$vh == ""} {set vh [winfo screenheight .]}
+    	 set x [lindex $g 2]
+    	 while {$x < 0} {incr x $vw}
+    	 while {$x > $vw} {incr x -$vw}
+    	 set y [lindex $g 3]
+    	 while {$y < 0} {incr y $vh}
+    	 while {$y > $vh} {incr y -$vh}
+    	 wm geometry $t +$x+$y
+    	 wm deiconify $t
     }
     update
 
@@ -131,7 +132,9 @@ proc dsk_bgexec {cmd label} {
     #eval blt_bgexec -errorvar dsk_bgexec(err$cnt) dsk_bgexec(out$cnt) $cmd
     eval blt_bgexec dsk_bgexec(stat$cnt) \
 	    -output dsk_bgexec(out$cnt) \
-	    -error dsk_bgexec(err$cnt) $cmd &
+	    -error dsk_bgexec(err$cnt) \
+	    -killsignal SIGINT \
+	    $args $cmd &
     tkwait variable dsk_bgexec(stat$cnt)
     
     set tkdesk(bgexec,working) 0
@@ -166,6 +169,8 @@ proc dsk_bgexec {cmd label} {
 
 set dsk_create(win,file) ""
 set dsk_create(win,directory) ""
+set dsk_create(open,file) 0
+set dsk_create(open,directory) 0
 
 proc dsk_create {type} {
     global tkdesk dsk_create
@@ -181,6 +186,7 @@ proc dsk_create {type} {
 		-label "Name of new file:" \
 		-title "New File" \
 		-checklabel "Open in Editor" \
+		-checkvalue $dsk_create(open,file) \
 		-callback {dsk_create_callback file} \
 		-nograb 1]
     } else {
@@ -188,7 +194,8 @@ proc dsk_create {type} {
 		-label "Name of new directory:" \
 		-title "New Directory" \
 		-checklabel "Open after Creation" \
-		-callback {dsk_create_callback dir} \
+		-checkvalue $dsk_create(open,directory) \
+		-callback {dsk_create_callback directory} \
 		-nograb 1]
     }
 }
@@ -212,6 +219,12 @@ proc dsk_create_callback {type name check} {
 	if {[string index $name 0] != "/"} {
 	    set name [string trimright [dsk_active dir] "/"]/$name
 	}
+	if [file exists $name] {
+	    dsk_errbell
+	    cb_error "$name already exists!"
+	    dsk_create $type
+	    return
+	}
 	set err [catch {eval exec $tkdesk(cmd,$cmd) [list $name]} errmsg]
 	if $err {
 	    dsk_errbell
@@ -222,6 +235,7 @@ proc dsk_create_callback {type name check} {
 	dsk_refresh [file dirname $name]
 	dsk_select file $name
 
+	set dsk_create(open,$type) $check
 	if $check {
 	    if {$type == "file"} {
 		dsk_edit $name
@@ -253,17 +267,17 @@ proc dsk_select {cmd args} {
 	    foreach fl [itcl_info objects -class dsk_FileListbox] {
 		if {$d == [string trimright \
 			[$fl info public directory -value] "/"]} {
-		    $fl.dlb select name $t
+		    [$fl getLbObj] select name $t
 		}
 	    }
 	}
 	X {
-	    if [winfo exists $tkdesk(active_viewer)] {
+	    if {[itcl_info objects $tkdesk(active_viewer)] != ""} {
 		$tkdesk(active_viewer) select X $args
 	    }
 	}
    	clear {
-	    if [winfo exists $tkdesk(active_viewer)] {
+	    if {[itcl_info objects $tkdesk(active_viewer)] != ""} {
 		$tkdesk(active_viewer) select clear
 	    }
 	}
@@ -399,8 +413,8 @@ proc dsk_check_perm {file} {
 # Side-FX:	none
 #
 
+
 set dsk_casdirs(menunum) 0
-set dsk_casdirs(cmax) [expr [winfo screenheight .]/21]
 
 proc dsk_casdirs {dir m {reset 0} {cmd ""} {mixed 0}} {
     global tkdesk dsk_casdirs tk_version
@@ -536,7 +550,7 @@ proc dsk_casdirs {dir m {reset 0} {cmd ""} {mixed 0}} {
 		}
 	    }
 	    incr c
-	    if {$c >= $dsk_casdirs(cmax)} {
+	    if {$c >= $tkdesk(_max_menu_entries)} {
 		set om $m
 		$m add cascade -label "More..." -menu [set m $m.c$c]
 		catch {destroy $m}
