@@ -67,6 +67,9 @@ proc dsk_find_reset_fields {} {
     set dsk_find(samefs) 0
     set dsk_find(follow) 0
     set dsk_find(case) 0
+    set dsk_find(caseFile) 1
+    set dsk_find(wordRegexp) 0
+    set dsk_find(doit) 0
 }
 
 dsk_find_reset_fields
@@ -78,6 +81,7 @@ proc dsk_find_files {args} {
 	dsk_find_reset_fields
 	set path_set [dsk_find_parse_args $args]
     } else {
+	set dsk_find(doit) 0
 	set path_set 0
     }
 
@@ -92,6 +96,10 @@ proc dsk_find_files {args} {
 	    }
 	}
 	cb_raise $t
+	update
+	if $dsk_find(doit) {
+	    dsk_do_find_files
+	}
 	return
     }
 
@@ -114,8 +122,10 @@ proc dsk_find_files {args} {
 	    set dsk_find(path) [string trimright $p "/"]
 	}
     }
-    bind $t.esp <Return> "$t.bSearch.button flash ; $t.bSearch.button invoke"
-    blt_drag&drop target $t.esp handler text "dd_handle_text $t.esp 1"
+    bind $t.esp <Return> "$t.bSearch.button invoke"
+    bind $t.esp <Escape> "$t.bClose invoke"
+    bind $t.esp <Control-c> "$t.bClose invoke; break"
+    blt_drag&drop target $t.esp handler file "dd_handle_text $t.esp %v 1"
     cb_bindForCompletion $t.esp <Control-Tab>
     cb_balloonHelp $t.esp "The search will start at this directory. If this field remains blank, search will start at the current directory of the last active file browser. You may specify multiple paths by separating them with a space."
     
@@ -146,9 +156,11 @@ proc dsk_find_files {args} {
     frame $t.f2
     pack $t.f2 -in $t.f -fill x -expand yes -padx $tkdesk(pad)
 
-    label $t.ln -text "File-Mask:" -width 10 -anchor w
+    label $t.ln -text "File-Masks:" -width 10 -anchor w
     entry $t.en -bd 2 -relief sunken -width 16 -textvar dsk_find(name)
-    bind $t.en <Return> "$t.bSearch.button flash ; $t.bSearch.button invoke"
+    bind $t.en <Return> "$t.bSearch.button invoke"
+    bind $t.en <Escape> "$t.bClose invoke"
+    bind $t.en <Control-c> "$t.bClose invoke; break"
     cb_balloonHelp $t.en "Enter a shell-like glob pattern here (e.g. *.c) to search for files with a certain name. You may specify multiple masks by separating them with a space."
     frame $t.fs2 -width 8
     tk_optionMenu $t.mst dsk_find(strtype) "String" "RegExp" "ExtReg"
@@ -156,8 +168,10 @@ proc dsk_find_files {args} {
     cb_balloonHelp $t.mst "Whether to look for a set of strings (separated by ||), for a regular expression, or for an extended regular expression (see the grep manual-page for more)."
     #label $t.lst -text "String:"
     entry $t.est -bd 2 -relief sunken -width 34 -textvar dsk_find(string)
-    bind $t.est <Return> "$t.bSearch.button flash ; $t.bSearch.button invoke"
-    cb_balloonHelp $t.est "Enter a regular expression in this field if you're looking for files containing a certain string."
+    bind $t.est <Return> "$t.bSearch.button invoke"
+    bind $t.est	<Escape> "$t.bClose invoke"
+    bind $t.est	<Control-c> "$t.bClose invoke; break"
+    cb_balloonHelp $t.est "Enter a string or regular expression in this field if you're looking for files containing a certain string."
     pack $t.ln $t.en $t.fs2 $t.mst $t.est \
 	    -in $t.f2 -side left -pady $tkdesk(pad)
     pack config $t.en -ipady 2
@@ -169,12 +183,16 @@ proc dsk_find_files {args} {
 
     label $t.lown -text "Owner:" -width 10 -anchor w
     entry $t.eown -bd 2 -relief sunken -width 16 -textvar dsk_find(owner)
-    bind $t.en <Return> "$t.bSearch.button flash ; $t.bSearch.button invoke"
+    bind $t.en <Return> "$t.bSearch.button invoke"
+    bind $t.en <Escape> "$t.bClose invoke"
+    bind $t.en <Control-c> "$t.bClose invoke; break"
     cb_balloonHelp $t.eown "Enter a numeric or textual user ID to look for files belonging to that user."
     frame $t.fs3b -width 8
     label $t.lgrp -text "Group:" -anchor w
     entry $t.egrp -bd 2 -relief sunken -width 16 -textvar dsk_find(group)
-    bind $t.egrp <Return> "$t.bSearch.button flash ; $t.bSearch.button invoke"
+    bind $t.egrp <Return> "$t.bSearch.button invoke"
+    bind $t.egrp <Escape> "$t.bClose invoke"
+    bind $t.egrp <Control-c> "$t.bClose invoke; break"
     cb_balloonHelp $t.egrp "Enter a numeric or textual group ID to look for files belonging to that group."
     frame $t.fs3b2 -width 8
     tk_optionMenu $t.mt dsk_find(type) "All Types" "Regular Files" \
@@ -193,7 +211,9 @@ proc dsk_find_files {args} {
 
     label $t.ls -text "Size (kB):" -width 10 -anchor w
     entry $t.es -bd 2 -relief sunken -width 16 -textvar dsk_find(size)
-    bind $t.es <Return> "$t.bSearch.button flash ; $t.bSearch.button invoke"
+    bind $t.es <Return> "$t.bSearch.button invoke"
+    bind $t.es <Escape> "$t.bClose invoke"
+    bind $t.es <Control-c> "$t.bClose invoke; break"
     cb_balloonHelp $t.es "If the size of files is important, this either gives the exact number of kilobytes, or, if preceded with a + resp. -, files will be matched which are bigger resp. smaller than this number. E.g. +100 will match all files which are bigger than 100 kB."
     frame $t.fs3 -width 8
     tk_optionMenu $t.mtm  dsk_find(time) "Last Accessed" "Last Modified"
@@ -201,13 +221,21 @@ proc dsk_find_files {args} {
     cb_balloonHelp $t.mtm "This setting relates to the entry field right to this menu button. The value entered there can either relate to when the file was last accessed or last modified."
     entry $t.etm -bd 2 -relief sunken -width 5 -textvar dsk_find(days)
     cb_balloonHelp $t.etm "10 will match files which are 10 days old, +10 will match all files older than 10 days, and -10 all newer than 10 days. Leave this field blank if this is not important for your search."
-    bind $t.etm <Return> "$t.bSearch.button flash ; $t.bSearch.button invoke"
+    bind $t.etm <Return> "$t.bSearch.button invoke"
+    bind $t.etm	<Escape> "$t.bClose invoke"
+    bind $t.etm	<Control-c> "$t.bClose invoke; break"
     frame $t.fs32 -width 8
     menubutton $t.mbOpts -text "Options..." -menu [set m $t.mbOpts.m] \
 	    -padx 4 -relief raised -bd 2
     cb_balloonHelp $t.mbOpts "This is a menubutton giving access to several options related to the find operation."
     menu $m
-    $m add checkbutton -label "Case Sensitive" -variable dsk_find(case)
+    $m add checkbutton -label "Case Sensitive" \
+	    -variable dsk_find(case)
+    $m add checkbutton -label "Whole words" \
+	    -variable dsk_find(wordRegexp)
+    $m add checkbutton -label "File-Mask Case Sensitive" \
+	    -variable dsk_find(caseFile)
+    $m add separator
     $m add checkbutton -label "Same Filesystem" -variable dsk_find(samefs)
     $m add checkbutton -label "Follow Symbolic Links" \
 	    -variable dsk_find(follow)
@@ -278,6 +306,10 @@ proc dsk_find_files {args} {
     update
     pack propagate $t 0
     wm deiconify $t
+
+    if $dsk_find(doit) {
+	dsk_do_find_files
+    }
 }
 
 # ----------------------------------------------------------------------------
@@ -375,12 +407,18 @@ proc dsk_find_parse_args {args} {
 	    -case* {
 		set dsk_find(case) 1
 	    }
+	    -word* {
+		set dsk_find(wordRegexp) 1
+	    }
 	    -same* {
 		set dsk_find(samefs) 1
 	    }
 	    -follow* {
 		set dsk_find(follow) 1
 	    }
+	    -doit {
+		set dsk_find(doit) 1
+	    }
 	}
     }
 
@@ -398,11 +436,7 @@ proc dsk_do_find_files {} {
     
     if {$dsk_find(path) != ""} {
 	foreach path $dsk_find(path) {
-	    if {$path != "/"} {
-		append cmd " \"[cb_tilde [string trimright $path /] expand]\""
-	    } else {
-		append cmd " $path"
-	    }
+	    append cmd " \"[cb_tilde [string trimright $path /]/ expand]\""
 	}
     } else {
 	set path [dsk_active dir]
@@ -436,6 +470,19 @@ proc dsk_do_find_files {} {
 	    append cmd " \\( "
 	    set do_or 0
 	    foreach n $dsk_find(name) {
+		if {!$dsk_find(caseFile)} {
+		    set n2 ""
+		    for {set i 0} {$i < [string length $n]} {incr i} {
+			set c [string tolower [string index $n $i]]
+			set uc [string toupper $c]
+			if {$c != $uc} {
+			    append n2 "\[$uc$c\]"
+			} else {
+			    append n2 $c
+			}
+		    }
+		    set n $n2
+		}
 		if $do_or {
 		    append cmd " -o -name \"$n\""
 		} else {
@@ -445,7 +492,21 @@ proc dsk_do_find_files {} {
 	    }
 	    append cmd " \\) "
 	} else {
-	    append cmd " -name $dsk_find(name)"
+	    set n $dsk_find(name)
+	    if {!$dsk_find(caseFile)} {
+		set n2 ""
+		for {set i 0} {$i < [string length $n]} {incr i} {
+		    set c [string tolower [string index $n $i]]
+		    set uc [string toupper $c]
+		    if {$c != $uc} {
+			append n2 "\[$uc$c\]"
+		    } else {
+			append n2 $c
+		    }
+		}
+		set n $n2
+	    }
+	    append cmd " -name \"[string trim $n \"]\""
 	}
     }
     if {$dsk_find(size) != ""} {
@@ -455,7 +516,9 @@ proc dsk_do_find_files {} {
 	}
 	set num [string trimleft $dsk_find(size) "+-"]
 	# 2 * $num, because -size argument is in blocks (512 bytes)
-	append cmd " -size $rc[expr 2 * $num]"
+	if {[string length $num] > 0} {
+	    append cmd " -size $rc[expr 2 * $num]"
+	}
     }
     if {$dsk_find(days) != ""} {
 	if {$dsk_find(time) == "Last Accessed"} {
@@ -495,6 +558,10 @@ proc dsk_do_find_files {} {
 		set grep "egrep -e"
 	    }
 	}
+	if $dsk_find(wordRegexp) {
+	    append grep " -w"
+	}
+	set str [dskC_esc $str { \"[]{}$;}]
 	if $dsk_find(case) {
 	    append cmd " -exec $grep -l \"$str\" \\\{\\\} \\\;"
 	} else {
@@ -503,7 +570,6 @@ proc dsk_do_find_files {} {
     } else {
 	append cmd " -print"
     }
-    append cmd " | sort"
 
     set cmd [string_replace $cmd \[ \\\[]
     set cmd [string_replace $cmd \] \\\]]
@@ -517,35 +583,67 @@ proc dsk_do_find_files {} {
     $t.llb config -text "Looking for matches..."
 
     # perform search (may take a long time)
-    dsk_busy $t
-    set matches [dsk_bgexec $cmd "Searching matches..."]
-    dsk_lazy $t
+    #dsk_busy $t
+    $t.bSearch.button config -state disabled
+    $t.bClose config -state disabled
+    
+    set dsk_find(lbl) ""
+    set matches [split [dsk_bgexec $cmd "Searching matches..." \
+	    -onoutput _find_update] "\n"]
+    dsk_debug $matches
 
     # OK, done with searching
-    if {$matches != "break"} {
-	set ml [split $matches \n]
+    if {$matches != "break" || $dsk_find(lbl) != ""} {
+	if {$matches != "break"} {
+	    set ml [lsort $matches]
+	    set abstr ""
+	} else {
+	    set ml [lsort $dsk_find(lbl)]
+	    set abstr "(search was abandoned)"
+	    dsk_debug $ml
+	}
 	set ll [llength $ml]
 
 	wm deiconify $t
 	raise $t
 	if {$ll == 1} {
-	    $t.llb config -text "Found 1 matching file:"
+	    $t.llb config -text "Found 1 matching file: $abstr"
 	} elseif {$ll == 0} {
 	    $t.llb config -text "No matches found."
 	} else {
-	    $t.llb config -text "Found $ll matching files:"
+	    $t.llb config -text "Found $ll matching files: $abstr"
 	}
 	set lbl ""
+	set mbl ""
 	foreach file $ml {
 	    if [file exists $file] {
 		lappend lbl [file dirname $file]/[subst \
 			[lindex [dskC_ls -l -o $file] 0]]
+		eval lappend lbl $mbl
+		set mbl ""
+	    } else {
+		lappend mbl "      $file"
 	    }
 	}
 	$t.dlb config -list $lbl
     } else {
 	catch {$t.llb config -text "Search abandoned."}
     }
+    
+    #dsk_lazy $t
+    $t.bSearch.button config -state normal
+    $t.bClose config -state normal
+}
+
+proc _find_update {data} {
+    global dsk_find
+
+    # Warning: $data may hold multiple file names separated by newlines!
+    eval lappend dsk_find(lbl) [split $data \n]
+    
+    #set file $data
+    #[file dirname $file]/[subst [lindex [dskC_ls -l -o $file] 0]]
+    .dsk_find_files.dlb config -list $dsk_find(lbl)
 }
 
 
@@ -561,69 +659,67 @@ proc _dsk_find_lb {name} {
     dsk_Listbox $name -width 10 -height 2 \
 	    -font [cb_font $tkdesk(font,file_lbs)] \
 	    -bg [cb_col $tkdesk(color,filelb)]
-    $name-frame config -relief flat
-    $name.text config -tabs {465 right 480 left 575 left  635 left 695 \
+    set frame [$name getFrame]
+    $frame config -relief flat
+    $frame.text config -tabs {465 right 480 left 575 left  635 left 695 \
 	    left 775 left 795 left}
     
-    bind $name.text <Any-Double-1> "_dsk_find_lb_open $name %s"
-    bind $name.text <3> "[bind $name.text <1>];\
+    bind $frame.text <Any-Double-1> "_dsk_find_lb_open $name %s"
+    bind $frame.text <3> "[bind $frame.text <1>];\
 	    _dsk_find_lb_popup $name %X %Y"
 
-    blt_drag&drop source $name.text config \
-	    -button 2 -packagecmd "find_dd_pkgcmd $name" \
+    blt_drag&drop source $frame.text  \
+	    -button 2 -packagecmd "find_dd_pkgcmd $name %t" \
 	    -rejectfg red -selftarget 0 -send {file text} \
 	    -tokenanchor nw \
 	    -tokencursor "@$tkdesk(library)/images/xbm/hand.xbm \
 	    $tkdesk(library)/images/xbm/hand.mask.xbm \
 	    [cb_col $tkdesk(color,foreground)] $tkdesk(color,drag)"
 
-    blt_drag&drop source $name.text handler \
-	    file dd_send_file text dd_send_text
+    blt_drag&drop source $frame.text handler file
 
-    bind $name.text <Shift-Button-2> "[bind Text <Button-2>]"
-    bind $name.text <Shift-B2-Motion> "[bind Text <B2-Motion>]"
-    bind $name.text <ButtonPress-2> "
+    bind $frame.text <Shift-Button-2> "[bind Text <Button-2>]"
+    bind $frame.text <Shift-B2-Motion> "[bind Text <B2-Motion>]"
+    bind $frame.text <ButtonPress-2> "
         $name _sel_for_dd @%x,%y
         $name _dd_start %X %Y
-        [bind $name.text <ButtonPress-2>]
+        blt_drag&drop drag %W %X %Y
     "
     
-    bind $name.text <Control-ButtonPress-2> \
-	    "[bind $name.text <ButtonPress-2>]"
+    bind $frame.text <Control-ButtonPress-2> \
+	    "[bind $frame.text <ButtonPress-2>]"
     
-    bind $name.text <Control-B2-Motion> \
+    bind $frame.text <Control-B2-Motion> \
 	    "set tkdesk(file_lb,control) 1 ;\
-	    [bind $name.text <B2-Motion>] ;\
 	    _dsk_find_dragcmd"
     
-    bind $name.text <Control-ButtonRelease-2> \
+    bind $frame.text <Control-ButtonRelease-2> \
 	    "set tkdesk(file_lb,control) 1 ;\
-	    [bind $name.text <ButtonRelease-2>]
-    dsk_desktop_drop %X %Y \[_dsk_find_get_sel $name\]
-    focus -force .
+ 	     blt_drag&drop drop %W %X %Y ;\
+             dsk_desktop_drop %X %Y \[_dsk_find_get_sel $name\]
+             focus -force .
     "
 
-    bind $name.text <ButtonRelease-2> "
+    bind $frame.text <ButtonRelease-2> "
     set tkdesk(file_lb,control) 0
-    [bind $name.text <ButtonRelease-2>]
+    blt_drag&drop drop %W %X %Y ;\
     dsk_desktop_drop %X %Y \[_dsk_find_get_sel $name\]
     focus -force .
     "
 
-    bind $name.text <B2-Motion> \
+    bind $frame.text <B2-Motion> \
 	    "set tkdesk(file_lb,control) 0;\
-	    [bind $name.text <B2-Motion>] ;\
 	    _dsk_find_dragcmd"
 
-    bind $name.text <B2-KeyRelease-Control_L> \
+    bind $frame.text <B2-KeyRelease-Control_L> \
 	    "set tkdesk(file_lb,control) 0; \
 	    _dsk_find_dragcmd"
     
-    bind $name.text <B2-KeyPress-Control_L> \
+    bind $frame.text <B2-KeyPress-Control_L> \
 	    "set tkdesk(file_lb,control) 1; \
 	    _dsk_find_dragcmd"
     
-    return $name
+    return $frame
 }
 
 proc _dsk_find_dragcmd {} {
@@ -651,7 +747,7 @@ proc _dsk_find_lb_open {dlb shift} {
 	set tmpfile [string trimright $tmpfile " "]
 	if {$tmpfile != ""} {
 	    set tkdesk(editor,findexp) $dsk_find(string)
-	    if {$dsk_find(strtype) == "Strings:"} {
+	    if {$dsk_find(strtype) == "String"} {
 		set tkdesk(editor,findisreg) 0
 	    } else {
 		set tkdesk(editor,findisreg) 1
@@ -664,13 +760,21 @@ proc _dsk_find_lb_open {dlb shift} {
 
 # binding for right click:
 proc _dsk_find_lb_popup {dlb x y} {
+    global tkdesk dsk_find
     
     set sel [$dlb select get]
     if {$sel != ""} {
 	set tmpfile [lindex [split [lindex [$dlb get] $sel] \t] 0]
 	set tmpfile [string trimright $tmpfile " "]
 	if {$tmpfile != ""} {
+	    set tkdesk(editor,findexp) $dsk_find(string)
+	    if {$dsk_find(strtype) == "Strings:"} {
+		set tkdesk(editor,findisreg) 0
+	    } else {
+		set tkdesk(editor,findisreg) 1
+	    }
 	    dsk_popup $dlb $tmpfile $x $y dir
+	    set tkdesk(editor,findexp) ""
 	}
     }
 }
@@ -728,23 +832,23 @@ proc find_dd_pkgcmd {dlb token} {
     }
     if {$ndirs > 0} {
         if {$ndirs == 1} {
-        	label $token.lDirs -text " 1 Directory "
+        	label $token.lDirs -text "  1 Directory "
         } else {
-        	label $token.lDirs -text " $ndirs Directories "
+        	label $token.lDirs -text "  $ndirs Directories "
         }
         pack $token.lDirs -anchor w
     }
     if {$nfiles > 0} {
         if {$nfiles == 1} {
-        	label $token.lFiles -text " 1 File "
+        	label $token.lFiles -text "  1 File "
         } else {
-        	label $token.lFiles -text " $nfiles Files "
+        	label $token.lFiles -text "  $nfiles Files "
         }
         pack $token.lFiles -anchor w
     }
 
     catch "wm deiconify $token"
     #update
-    return $flist
+    return [list $flist]
 }
 
