@@ -72,7 +72,8 @@ proc dsk_about {} {
     label $t.lTkDesk -image welcome_bm
     pack $t.lTkDesk -in $t.f1 -padx 6 -pady 2 -fill x -expand yes
     #message $t.lVersion -text "Version $tkdesk(version), dated $tkdesk(date)\nCopyright (C) 1996, 1997 Christian Bolik\n\nTkDesk comes with ABSOLUTELY NO WARRANTY.\nSee menu entry Help/License for details." -aspect 1000 -justify center
-    message $t.lVersion -text "A File Manager for\nUnix and the X Window System\n\nVersion $tkdesk(version), dated $tkdesk(date)\n\nWritten by Christian Bolik\n($tkdesk(authormail))\n\nPlease send me a short mail and tell me what you\nthink about this program, and what you feel could be\nimproved or added. Thanks!\n" -aspect 1000 -justify center
+    #message $t.lVersion -text "A File Manager for\nUnix and the X Window System\n\nVersion $tkdesk(version), dated $tkdesk(date)\n\nWritten by Christian Bolik\n($tkdesk(authormail))\n\nPlease send me a short mail and tell me what you\nthink about this program, and what you feel could be\nimproved or added. Thanks!\n" -aspect 1000 -justify center
+    message $t.lVersion -text "A File Manager for\nUnix and the X Window System\nWritten by Christian Bolik\n\nVersion $tkdesk(version), dated $tkdesk(date)\n\nTkDesk is maintained by J. Chris Coppick\n($tkdesk(maintmail))\n\nPlease send a short email about what you\nthink about this program, and what you feel could be\nimproved or added. Thanks!\n" -aspect 1000 -justify center
     catch {$t.lVersion config -font -*-helvetica-medium-r-*-*-14-*}
     pack $t.lVersion -in $t.f1 -padx 8 -pady 8 -fill x -expand yes
 
@@ -83,7 +84,7 @@ proc dsk_about {} {
     pack $t.fBottom -in $t.f
     
     cb_button $t.bOK -text "OK" -width 6 -command "destroy $t" -default 1
-    bind $t <Return> "destroy $t"
+    bind $t <Return> "if \[winfo exists $t\] \{destroy $t\}"
     cb_button $t.bLic -text " License " -command "dsk_help license"
     cb_button $t.bMail -text " Mail... " \
 	    -command "dsk_mailbug \"About TkDesk $tkdesk(version)...\" {}"
@@ -203,3 +204,51 @@ proc dsk_help {what} {
     }
     dsk_lazy
 }
+
+
+proc dsk_about_web {} {
+    global tkdesk
+
+    set t .dsk_about
+    
+    if [winfo exists $t] {
+	cb_raise $t
+	dsk_sound dsk_about
+	return
+    }
+
+    toplevel $t
+    wm withdraw $t
+
+    frame $t.f -bd 2 -relief raised
+    pack $t.f
+
+    frame $t.f1 -bd 2 -relief groove
+    pack $t.f1 -in $t.f -padx 8 -pady 8
+
+    label $t.lLabel -text "Presenting..." -anchor center
+    catch {$t.lLabel config -font -*-helvetica-medium-o-*-*-14-*}
+    pack $t.lLabel -in $t.f1 -padx 8 -pady 8 -fill x -expand yes
+
+    image create bitmap welcome_bm -data $tkdesk(welcome_bm) -foreground blue3
+    label $t.lTkDesk -image welcome_bm
+    pack $t.lTkDesk -in $t.f1 -padx 6 -pady 2 -fill x -expand yes
+    #message $t.lVersion -text "Version $tkdesk(version), dated $tkdesk(date)\nCopyright (C) 1996, 1997 Christian Bolik\n\nTkDesk comes with ABSOLUTELY NO WARRANTY.\nSee menu entry Help/License for details." -aspect 1000 -justify center
+    message $t.lVersion -text "A File Manager for\nUnix and the X Window System\n\nVersion $tkdesk(version), dated $tkdesk(date)" -aspect 1000  -justify center
+    catch {$t.lVersion config -font -*-helvetica-medium-o-*-*-14-*}
+    pack $t.lVersion -in $t.f1 -padx 8 -pady 8 -fill x -expand yes
+    
+    update
+    $t.lTkDesk config -width 280
+
+    frame $t.fButtons
+    pack $t.fButtons -in $t.f -fill x -padx 4
+
+    wm title $t ""
+    wm iconname $t "About TkDesk"
+    wm resizable $t 0 0
+    cb_centerToplevel $t
+    wm deiconify $t
+
+    dsk_sound dsk_about
+}
\ No newline at end of file
