@@ -51,6 +51,7 @@ proc dsk_refresh {files} {
 
     set refreshed_dirs ""
 
+    dsk_debug "Refreshing $files"
     foreach dir $files {
 	if ![file isdirectory $dir] {
 	    set dir [file dirname $dir]
@@ -75,6 +76,10 @@ proc dsk_refresh {files} {
 	    if {$dir == "$tkdesk(trashdir)/"} {
 		_appbar_trash_refresh
 	    }
+	    
+	    if {$dir == $tkdesk(deskdir)} {
+		dsk_desktop_update
+	    }
 	}
     }
 
@@ -97,15 +102,22 @@ proc dsk_refresh {files} {
 
 set tkdesk(bgexec,working) 0
 
-proc dsk_update_flbs {} {
+proc dsk_update_flbs {{lblist {}}} {
     global tkdesk
 
-    # dsk_debug "dsk_update_flbs entry"
+    #dsk_debug "dsk_update_flbs entry, lblist: $lblist"
     set refreshed_toplevels ""
 
+    if {[llength $lblist] == 0} {
+	set lblist [itcl_info objects -class dsk_FileListbox]
+	set revisit 1
+    } else {
+	set revisit 0
+    }
+
     set err [catch {
 	if {!$tkdesk(bgexec,working)} {
-	    foreach lb [itcl_info objects -class dsk_FileListbox] {
+	    foreach lb $lblist {
 		set dir ""
 		catch {set dir [$lb info public directory -value]}
 		if {$dir == ""} {
@@ -122,17 +134,39 @@ proc dsk_update_flbs {} {
 			    #$lb config -directory $dir
 			    $lb refresh
 			} else {
-			    [winfo toplevel $lb] refreshdir $dir
+			    [$lb cget -viewer] refreshdir $dir
 			}
 		    }
 		} else {
-		    set toplevel [winfo toplevel $lb]
+		    set toplevel [$lb cget -viewer]
 		    if {[lsearch $refreshed_toplevels $toplevel] < 0} {
 			$toplevel config -directory [_make_path_valid $dir]
 			lappend refreshed_toplevels $toplevel
 		    }
 		}
 	    }
+	    
+	    if $tkdesk(auto_reload_conf) {
+		set mt [file mtime $tkdesk(configdir)]
+		if {$mt > $tkdesk(configdir_mtime)} {
+		    #puts "Setting configdir_mtime to $mt"
+		    set tkdesk(configdir_mtime) $mt
+		    foreach cf $tkdesk(configfiles) {
+			set fmt [file mtime $tkdesk(configdir)/$cf]
+			#puts "Comparing $cf: $fmt - $mt"
+			if {$fmt >= $mt} {
+			    #puts "Reloading $cf"
+			    dsk_reread_config $cf
+			}
+		    }
+		}
+	    }
+	    
+	    set mt [file mtime $tkdesk(deskdir)]
+	    if {$mt > $tkdesk(deskdir_mtime)} {
+		set tkdesk(deskdir_mtime) $mt
+		dsk_desktop_update
+	    }
 	}
     } errmsg]
 
@@ -140,9 +174,11 @@ proc dsk_update_flbs {} {
 	puts stderr $errmsg
     }
 
-    set tkdesk(afterid,update_flbs) \
-	    [after [expr int($tkdesk(update,file_lists) * 1000)] \
-	    dsk_update_flbs]
-    # dsk_debug "dsk_update_flbs exit"
+    if {$revisit} {
+	set tkdesk(afterid,update_flbs) \
+		[after [expr int($tkdesk(update,file_lists) * 1000)] \
+		dsk_update_flbs]
+    }
+    #dsk_debug "dsk_update_flbs exit"
 }
 
