@@ -33,7 +33,13 @@
 static char sccsid[] = "@(#) tkAppInit.c 1.12 94/12/17 16:30:56";
 #endif
 */
+#include "config.h"
+#ifdef HAVE_TCL8_0_H
+#include "tcl8.0.h"
+#include "tk8.0.h"
+#else
 #include "tk.h"
+#endif
 #include "libdesk/ot.h"
 
 #ifdef STATIC_BUILD
@@ -102,9 +108,11 @@ main(argc, argv)
 	    fprintf (stderr, "contains TkDesk's library files (such as Common.tcl).\n");
 	    exit (2);
 	} else {
-	    sprintf (buf[0], "TCL_LIBRARY=%s/tcl_lib", libbuf);
+	    sprintf (buf[0], "TCL_LIBRARY=%s/static/tcl8.0", libbuf);
 	    putenv (buf[0]);
-	    sprintf (buf[1], "TK_LIBRARY=%s/tk_lib", libbuf);
+	    sprintf (buf[1], "TK_LIBRARY=%s/static/tk8.0", libbuf);
+	    putenv (buf[1]);
+	    sprintf (buf[1], "ITCL_LIBRARY=%s/static/itcl3.0", libbuf);
 	    putenv (buf[1]);
 	}
     }
